/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
2003 Erik Bengtson  - removed ununsed imports
2003 Andy Jefferson - commented
    ...
**********************************************************************/
package org.jpox;

import javax.jdo.PersistenceManagerFactory;
import javax.jdo.datastore.JDOConnection;

/**
 * Implementation of the JDO PersistenceManager.
 *
 * @version $Revision: 1.4 $
 **/
public class PersistenceManagerImpl
    extends AbstractPersistenceManager
    implements PersistenceManager
{
    /**
     * Constructor.
     * @param pmf The Persistence Manager Factory
     * @param userName The user login
     * @param password The password for the supplied login
     **/
    public PersistenceManagerImpl(AbstractPersistenceManagerFactory pmf, String userName, String password)
    {
        super(pmf, userName, password);
        tx = getStoreManager().getNonmanagedTransaction(this, userName, password);
    }

    /**
     * Accessor for the Persistence Manager Factory.
     * @return The Persistence Manager factory
     **/
    public PersistenceManagerFactory getPersistenceManagerFactory()
    {
        return (PersistenceManagerFactory)getAbstractPersistenceManagerFactory();
    }

    /**
     * Accessor for a handle to the Persistence Manager
     * @return The PM handle.
     **/
    public PersistenceManager getPMHandle()
    {
        return this;
    }

    /**
     * Accessor for a connection on the datastore.
     * See JDO 2.0 spec section 12.16
     * @return The JDO connection to the datastore
     * @see javax.jdo.PersistenceManager#getDataStoreConnection()
     * @since 1.1
     */
    public JDOConnection getDataStoreConnection()
    {
        return tx.getJDOConnection();
    }
}
