/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
2003 Andy Jefferson - commented
    ...
**********************************************************************/
package org.jpox;

import javax.jdo.datastore.JDOConnection;

/**
 * Extension of SUN's JDO Transaction including features general to all datastores.
 *
 * @version $Revision: 1.11 $
 **/ 
public interface Transaction extends javax.jdo.Transaction
{
    /**
     * Checks whether a transaction is committing.
     * @return Whether the transaction is committing
     */
    boolean isCommitting();

    /**
     * Accessor for the connection to the datasource.
     * If this method is called while a datastore transaction is active,
     * the object returned will be enlisted in the current transaction.
     * If called in an optimistic transaction or outside an active
     * transaction, the object returned will not be enlisted in any transaction.
     * 
     * The object must be returned to the JDO implementation prior to calling
     * any JDO method or performing any action on any persistent instance that
     * might require the JDO implementation to use a connection. If the object
     * has not been returned and the JDO implementation needs a connection,
     * a JDOUserException is thrown. The object is returned to the JDO
     * implementation by calling the standard method on the object.
     * 
        //TODO understand better the JDO spec
        //TODO if the object not enlisted is not closed, raise an exception?
     * @return The Connection
     */
    JDOConnection getJDOConnection();    
}