/**********************************************************************
Copyright (c) 2006 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.jpox.datasource;

import javax.sql.DataSource;

import org.jpox.ClassLoaderResolver;
import org.jpox.util.Localiser;

/**
 * Interface for a DataSourceFactory for use within JPOX.
 * 
 * @version $Revision: 1.3 $
 */
public interface JPOXDataSourceFactory
{
    /** Localisation of messages. */
    public static final Localiser LOCALISER = Localiser.getInstance("org.jpox.Localisation");

    /**
     * Method to make a DataSource for use within JPOX.
     * @param dbDriver The database driver name
     * @param dbURL The database URL
     * @param dbUser The database username
     * @param dbPassword The database password
     * @param configFile Location of any configuration properties file
     * @param clr ClassLoaderResolver to use
     * @return The DataSource
     * @throws Exception Thrown if an error occurs during creation
     */
    public DataSource makePooledDataSource(String dbDriver,
                                           String dbURL,
                                           String dbUser,
                                           String dbPassword,
                                           String configFile,
                                           ClassLoaderResolver clr);
}