/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
    ...
**********************************************************************/
package org.jpox.exceptions;

import javax.jdo.JDOUserException;

import org.jpox.util.Localiser;

/**
 * An <tt>MetaDataForPersistenceCapableClassNotReachableException</tt> is
 * thrown if a class implements PersistenceCapable interface but its jdo
 * metadata is unreachable in the classpath.
 * 
 * @version $Revision: 1.3 $
 * @see org.jpox.metadata.ClassMetaData
 */
public class MetaDataForPersistenceCapableClassNotReachableException extends JDOUserException
{
    private static final Localiser LOCALISER = Localiser.getInstance("org.jpox.Localisation");

    /**
     * Constructs a MetaData for PersistenceCapable class not reachable exception with the specified
     * detail message.
     * @param className Name of the class
     */
    public MetaDataForPersistenceCapableClassNotReachableException(String className)
    {
        super(LOCALISER.msg("Exception.ClassHasNoMetaData",className));
    }

    /**
     * Constructs a MetaData for PersistenceCapable class not reachable exception with the specified
     * detail message and nested exception.
     * @param className Name of the class
     * @param nested the nested exception(s).
     */
    public MetaDataForPersistenceCapableClassNotReachableException(String className, Exception nested)
    {
        super(LOCALISER.msg("Exception.ClassHasNoMetaData",className), nested);
    }
}