/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * Representation of the Meta-Data for a container.
 * This is subclasses by Array, Collection, and Map. 
 *
 * @version $Revision: 1.3 $
 */
public class ContainerMetaData extends ExtendableMetaData
{
    /**
     * Constructor.
     * @param parent The parent Field
     */
    public ContainerMetaData(AbstractPropertyMetaData parent)
    {
        super(parent);
    }

    /**
     * Accessor for the parent Field MetaData.
     * @return Parent metadata
     */
    public AbstractPropertyMetaData getAbstractPropertyMetaData()
    {
        if (parent != null)
        {
            return (AbstractPropertyMetaData)parent;
        }
        return null;
    }

    /**
     * Accessor for the parent field name
     * @return Parent field name.
     */
    public String getFieldName()
    {
        if (parent != null)
        {
            return ((AbstractPropertyMetaData)parent).getName();
        }
        return null;
    }

    /**
     * Accessor for the MetaDataManager of the file.
     * @return MetaDataManager.
     */
    public MetaDataManager getMetaDataManager()
    {
        if (parent != null)
        {
            return getAbstractPropertyMetaData().getAbstractClassMetaData().getMetaDataManager();
        }
        return null;
    }
}
