/**********************************************************************
Copyright (c) 2006 Erik Bengtson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
    ...
**********************************************************************/
package org.jpox.plugin;

import java.net.URL;

/**
 * A Plug-in definition. Represents the XML declaration 
 * @version $Revision: 1.3 $
 */
public class Plugin
{
    /** unique id **/
    final private String id;
    
    /** provider name **/
    final private String providerName;

    /** name **/
    final private String name;
    
    /** user friendly name **/
    final private String version;
    
    /** location of the plugin.xml file **/
    final private URL pluginLocation;

    /**
     * Constructor 
     * @param id the unique id
     * @param name the name
     * @param providerName the provider name
     * @param version the version number
     * @param pluginLocation the path to the declaration file
     */
    public Plugin(String id, String name, String providerName, String version, URL pluginLocation)
    {
        this.id = id;
        this.name = name;
        this.providerName = providerName;
        this.version = version;
        this.pluginLocation = pluginLocation;
    }
    
    /**
     * Accessor for the plug-in id
     * @return id of the plug-in
     */
    public String getId()
    {
        return id;
    }

    /**
     * Accessor for the provider name of this plug-in
     * @return provider name
     */
    public String getProviderName()
    {
        return providerName;
    }
    
    /**
     * Acessor for the version of this plug-in
     * @return version
     */
    public String getVersion()
    {
        return version;
    }
    
    /**
     * Acessor for the location of the plugin.xml file
     * @return the plugin.xml location
     */
    public URL getPluginLocation()
    {
        return pluginLocation;
    }
    
    /**
     * Acessor for the plug-in name
     * @return plug-in name
     */
    public String getName()
    {
        return name;
    }
}
