/**********************************************************************
Copyright (c) 2006 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
    ...
**********************************************************************/
package org.jpox.sco;

import org.jpox.state.FetchPlanState;

/**
 * Representation of a SCO that contains other (persistable) objects.
 *
 * @version $Revision: 1.1 $
 **/
public interface SCOContainer extends SCO
{
    /**
     * Inform the SCO that it should load itself fully now (in case it is using lazy loading).
     */
    void load();

    /**
     * Method to run reachability from this SCO.
     * @param reachables List of StateManagers reachable so far
     */
    void runReachability(java.util.Set reachables);

    /**
     * Method to make the object transient.
     * Makes transient all components of this object that are also PersistenceCapable.
     * @param state State of the fetchplan process
     */
    void makeTransient(FetchPlanState state);

    /**
     * Method to load all fields in the fetch plan (of all subobjects).
     * @param state The FetchPlanState
     */
    void loadFieldsInFetchPlan(FetchPlanState state);

    /**
     * Method to detach this object.
     * Detaches all components of this object that are also PersistenceCapable.
     * @param state State for the detachment process
     */
    void detach(FetchPlanState state);
}