/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
2002 Kelly Grizzle (TJDO)
2003 Andy Jefferson - commented
    ...
**********************************************************************/
package org.jpox.state;

import javax.jdo.JDOFatalInternalException;

import org.jpox.util.Localiser;
import org.jpox.StateManager;

/**
 * A <tt>IllegalStateTransitionException</tt> is thrown if a life-cycle
 * state attempts a transition that is illegal.  This indicates a coding
 * bug in the JDO implementation.
 *
 * @version $Revision: 1.3 $ 
 */
public class IllegalStateTransitionException extends JDOFatalInternalException
{
    /** Localiser for messages. */
    protected static final Localiser LOCALISER = Localiser.getInstance("org.jpox.Localisation");

    /**
     * Constructs an illegal state transition exception.
     * @param state The object's current life-cycle state.
     * @param transition A string describing the type of transition.
     * @param sm The StateManagerImpl of the object.
     */
    public IllegalStateTransitionException(LifeCycleState state, String transition, StateManager sm)
    {
        super(LOCALISER.msg("StateManager.IllegalStateTransition", transition, state, sm));
    }
}