/**********************************************************************
Copyright (c) 2006 Erik Bengtson and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 
 

Contributors:
    ...
**********************************************************************/
package org.jpox.state;

import java.util.Collection;
import java.util.Map;

import javax.jdo.spi.PersistenceCapable;
import javax.jdo.spi.StateManager;

import org.jpox.store.fieldmanager.AbstractFieldManager;

/**
 * Manager that nullifies any Collection/Map/PC fields of the object.
 * 
 * @version $Revision: 1.3 $
 */
public class NullifyRelationFieldManager extends AbstractFieldManager
{
    /** State Manager for the object. */
    private final StateManager sm;

    /**
     * Constructor.
     * @param sm the StateManager
     */
    public NullifyRelationFieldManager(StateManager sm)
    {
        this.sm = sm; 
    }

    public Object fetchObjectField(int fieldNumber)
    {
        Object object = ((StateManagerImpl)sm).provideField(fieldNumber);
        if (object == null)
        {
            return null;
        }
        else if (object instanceof Collection)
        {
            ((StateManagerImpl)sm).makeDirty(fieldNumber);
            ((Collection)object).clear();
            return object;
        }
        else if (object instanceof Map)
        {
            ((StateManagerImpl)sm).makeDirty(fieldNumber);
            ((Map)object).clear();
            return object;
        }
        else if (object instanceof PersistenceCapable)
        {
            ((StateManagerImpl)sm).makeDirty(fieldNumber);
            return null;
        }
        //do not need to nullify fields that are not references and resides embedded in this object
        return object;
    }

    public boolean fetchBooleanField(int fieldNumber)
    {
        return true;
    }
    
    public char fetchCharField(int fieldNumber)
    {
        return '0';
    }
    
    public byte fetchByteField(int fieldNumber)
    {
        return (byte)0;
    }
    
    public double fetchDoubleField(int fieldNumber)
    {
        return 0;
    }
    
    public float fetchFloatField(int fieldNumber)
    {
        return 0;
    }
    
    public int fetchIntField(int fieldNumber)
    {
        return 0;
    }
    
    public long fetchLongField(int fieldNumber)
    {
        return 0;
    }
    
    public short fetchShortField(int fieldNumber)
    {
        return 0;
    }
    
    public String fetchStringField(int fieldNumber)
    {
        return "";
    }
}
