/**********************************************************************
Copyright (c) 2005 Erik Bengtson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.jpox.store;

import java.util.Map;
import java.util.WeakHashMap;

/**
 * Factory for OID instances.
 * This factory holds a cache of weak references to OID. 
 * @version $Revision: 1.3 $
 */
public class OIDFactory
{
    private static Map oids = new WeakHashMap();

    private OIDFactory()
    {
        // Private constructor to prevent instantiation
    }

    /**
     * Factory method for OID instances
     * @param className the pc class name
     * @param value the id value
     * @return an OID instance
     */
    public static OID getInstance(String className, Object value)
    {
        String key = className + "#" + value.toString();
        OID oid = (OID) oids.get(key);
        if (oid == null)
        {
            oid = new OID(className,value);
            oids.put(key,oid);
        }
        return oid;
    }
}