/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 
 

Contributors:
2003 Andy Jefferson - introduction of localiser
2004 Andy Jefferson - moved to org.jpox.store
2006 Andy Jefferson - generalised so that can be used for any store
    ...
**********************************************************************/
package org.jpox.store;

import org.jpox.AbstractPersistenceManagerFactory;
import org.jpox.ClassLoaderResolver;
import org.jpox.util.ClassUtils;

/**
 * Factory for creating StoreManagers.
 *
 * @version $Revision: 1.8 $ 
 **/
public class StoreManagerFactory
{
    /**
     * Default Constructor.
     **/
    private StoreManagerFactory()
    {
        // protects this class from been instantiated
    }

    /**
     * Accessor for the StoreManager. Creates an instance of StoreManager if necessary
     * @param managerClassName Name of the class name for the Store Manager
     * @param clr the ClassLoaderResolver
     * @param pmf Persistence Manager Factory
     * @param userName The user name for this store
     * @param password The password for this store.
     * @return The Store Manager.
     **/
    public static synchronized StoreManager getStoreManager(String managerClassName,
            ClassLoaderResolver clr, 
            AbstractPersistenceManagerFactory pmf,
            String userName,
            String password)
    {
        if (userName == null)
        {
            userName = pmf.getPMFConfiguration().getConnectionUserName();
            password = pmf.getPMFConfiguration().getConnectionPassword();
        }
        String key = managerClassName + ';' + userName;

        // Check the cache for a store manager for this datastore
        StoreManager storeMgr = (StoreManager)pmf.getPMFContext().getStoreMgrs().get(key);
        if (storeMgr == null)
        {
            // Create a new StoreManager of this type
            Class managerCls = clr.classForName(managerClassName,AbstractPersistenceManagerFactory.class.getClassLoader());
            Class[] ctrArgTypes = new Class[] {ClassLoaderResolver.class, AbstractPersistenceManagerFactory.class, String.class, String.class};
            Object[] ctrArgs = new Object[] {clr, pmf, userName, password};
            storeMgr = (StoreManager)ClassUtils.newInstance(managerCls, ctrArgTypes, ctrArgs);

            
            // Store this manager in the cache
            pmf.getPMFContext().getStoreMgrs().put(key, storeMgr);
        }

        return storeMgr;
    }
}