/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.fieldmanager;

import javax.jdo.JDOFatalInternalException;

/**
 * Abstract representation of a field manager.
 *
 * @version $Revision: 1.3 $
 **/
public abstract class AbstractFieldManager implements FieldManager
{
    /**
     * Default constructor
     */
    public AbstractFieldManager()
    {
        //default constructor
    }

    private String failureMessage(String method)
    {
        return "Somehow " + getClass().getName() + "." + method + "() was called, which should have been impossible";
    }

    public void storeBooleanField(int fieldNumber, boolean value)
    {
        throw new JDOFatalInternalException(failureMessage("storeBooleanField"));
    }

    public boolean fetchBooleanField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchBooleanField"));
    }

    public void storeCharField(int fieldNumber, char value)
    {
        throw new JDOFatalInternalException(failureMessage("storeCharField"));
    }

    public char fetchCharField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchCharField"));
    }

    public void storeByteField(int fieldNumber, byte value)
    {
        throw new JDOFatalInternalException(failureMessage("storeByteField"));
    }

    public byte fetchByteField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchByteField"));
    }

    public void storeShortField(int fieldNumber, short value)
    {
        throw new JDOFatalInternalException(failureMessage("storeShortField"));
    }

    public short fetchShortField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchShortField"));
    }

    public void storeIntField(int fieldNumber, int value)
    {
        throw new JDOFatalInternalException(failureMessage("storeIntField"));
    }

    public int fetchIntField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchIntField"));
    }

    public void storeLongField(int fieldNumber, long value)
    {
        throw new JDOFatalInternalException(failureMessage("storeLongField"));
    }

    public long fetchLongField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchLongField"));
    }

    public void storeFloatField(int fieldNumber, float value)
    {
        throw new JDOFatalInternalException(failureMessage("storeFloatField"));
    }

    public float fetchFloatField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchFloatField"));
    }

    public void storeDoubleField(int fieldNumber, double value)
    {
        throw new JDOFatalInternalException(failureMessage("storeDoubleField"));
    }

    public double fetchDoubleField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchDoubleField"));
    }

    public void storeStringField(int fieldNumber, String value)
    {
        throw new JDOFatalInternalException(failureMessage("storeStringField"));
    }

    public String fetchStringField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchStringField"));
    }

    public void storeObjectField(int fieldNumber, Object value)
    {
        throw new JDOFatalInternalException(failureMessage("storeObjectField"));
    }

    public Object fetchObjectField(int fieldNumber)
    {
        throw new JDOFatalInternalException(failureMessage("fetchObjectField"));
    }
}
