/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.math.BigInteger;

import org.jpox.ClassLoaderResolver;
import org.jpox.store.expression.IntegerLiteral;
import org.jpox.store.expression.LogicSetExpression;
import org.jpox.store.expression.NumericExpression;
import org.jpox.store.expression.QueryExpression;
import org.jpox.store.expression.ScalarExpression;

/**
 * Mapping for BigInteger type.
 *
 * @version $Revision: 1.13 $
 **/
public class BigIntegerMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
	private static BigInteger mappingSampleValue = new BigInteger("0");

	public Object getSampleValue(ClassLoaderResolver clr)
	{
		return mappingSampleValue;
	}
	
    public ScalarExpression newLiteral(QueryExpression qs, Object value)
    {
        ScalarExpression expr = new IntegerLiteral(qs, this, (Number)value);
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryExpression qs, LogicSetExpression te)
    {
        ScalarExpression expr = new NumericExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return BigInteger.class;
    }
}
