/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 
 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - coding standards
2004 Erik Bengtson - included a check "is null" on getObject method 
    ...
**********************************************************************/
package org.jpox.store.mapping;

import org.jpox.ClassLoaderResolver;
import org.jpox.store.expression.CharacterExpression;
import org.jpox.store.expression.CharacterLiteral;
import org.jpox.store.expression.LogicSetExpression;
import org.jpox.store.expression.NumericExpression;
import org.jpox.store.expression.QueryExpression;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.rdbms.mapping.DecimalRDBMSMapping;
import org.jpox.store.rdbms.mapping.IntegerRDBMSMapping;
import org.jpox.store.rdbms.mapping.NumericRDBMSMapping;
import org.jpox.store.rdbms.mapping.SmallIntRDBMSMapping;

/**
 * Mapping for Character type.
 * In RDBMS, this mapping can be stored in INT or CHAR columns.
 * The use of INT columns facilitates greater than, less than operations within queries etc. 
 *
 * @version $Revision: 1.16 $
 **/
public class CharacterMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    private static Character mappingSampleValue = new Character('0');

    public Object getSampleValue(ClassLoaderResolver clr)
    {
        return mappingSampleValue;
    }

    public ScalarExpression newLiteral(QueryExpression qs, Object value)
    {
        ScalarExpression expr = new CharacterLiteral(qs, this, ((Character)value).toString());
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryExpression qs, LogicSetExpression te)
    {
        //TODO this code should not exist, or be moved to DatastoreMapping classes
        if( getDataStoreMapping(0) instanceof IntegerRDBMSMapping ||
            getDataStoreMapping(0) instanceof NumericRDBMSMapping ||
            getDataStoreMapping(0) instanceof SmallIntRDBMSMapping ||
            getDataStoreMapping(0) instanceof DecimalRDBMSMapping )
        {
            ScalarExpression expr = new NumericExpression(qs, this, te);
            return expr; 
        }
        ScalarExpression expr = new CharacterExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return Character.class;
    }

    /**
     * Method to return the default length of this type in the datastore.
     * Character(char will need a single character!
     * @param index The index position
     * @return The default length
     */
    public int getDefaultLength(int index)
    {
        return 1;
    }
}