/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 
 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.poid;

import java.io.Serializable;

/**
 * Persistent Object id. A general purpose oid. The OID can be of any
 * kind, String, Long, Date, Application ID, etc.
 * 
 * @version $Revision: 1.3 $
 */
class Poid implements Serializable
{
    /**
     * the oid
     */
    private Object oid;
 
    /**
     * Constructor for Poid
     * @param oid The Object ID
     */
    Poid(Object oid)
    {
        this.oid = oid;
    }

    /**
     * Returns the oid.
     * @return The OID
     */
    public Object getOid()
    {
        return oid;
    }

    /**
     * Sets the oid.
     * @param oid The oid to set
     */
    public void setOid(Object oid)
    {
        this.oid = oid;
    }

    /**
     * @param obj The object to compare against
     * @see java.lang.Object#equals(Object)
     * @return boolean the result of the comparation
     */
    public boolean equals(Object obj)
    {
        if (!(obj instanceof Poid))
        {
            return false;
        }
        return ((Poid)obj).getOid().equals(this.getOid());
    }
}
