/**********************************************************************
Copyright (c) 2003 Mike Martin (TJDO) and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 
 

Contributors:
2003 Kelly Grizzle (TJDO)
2003 Erik Bengtson  - removed exist() operation
2003 Erik Bengtson  - refactored the persistent id generator System property
2003 Andy Jefferson - added localiser
2003 Andy Jefferson - updated exception handling with SchemaTable
2003 Andy Jefferson - restructured to remove SchemaTable, and add StoreManagerHelper
2003 Andy Jefferson - updated getSubClassesForClass to recurse
2004 Erik Bengtson  - removed unused method and variables 
2004 Erik Bengtson  - fixed problem with getObjectById for App ID in getClassForOID
2004 Andy Jefferson - re-emergence of SchemaTable. Addition of addClass().
2004 Andy Jefferson - Addition of AutoStartMechanism interface
2004 Andy Jefferson - Update to use Logger
2004 Andy Jefferson - Addition of Catalog name to accompany Schema name
2004 Marco Schulze  - replaced catch(NotPersistenceCapableException ...)
                  by advance-check via TypeManager.isSupportedType(...)
2004 Andy Jefferson - split StoreData into superclass.
2004 Andy Jefferson - added support for other inheritance types
2004 Andy Jefferson - added capability to dynamically add columns
2005 Marco Schulze - prevent closing starter during recursion of ClassAdder.addClassTables(...)
    ...
**********************************************************************/
package org.jpox.store.rdbms;

import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import javax.jdo.Extent;
import javax.jdo.JDODataStoreException;
import javax.jdo.JDOException;
import javax.jdo.JDOFatalException;
import javax.jdo.JDOFatalInternalException;
import javax.jdo.JDOFatalUserException;
import javax.jdo.JDOUserException;
import javax.jdo.datastore.Sequence;
import javax.jdo.identity.SingleFieldIdentity;
import javax.resource.spi.security.PasswordCredential;

import org.jpox.AbstractPersistenceManagerFactory;
import org.jpox.ClassLoaderResolver;
import org.jpox.PersistenceManager;
import org.jpox.PersistenceManagerImpl;
import org.jpox.Transaction;
import org.jpox.exceptions.MetaDataForPersistenceCapableClassNotReachableException;
import org.jpox.metadata.AbstractClassMetaData;
import org.jpox.metadata.AbstractPropertyMetaData;
import org.jpox.metadata.ClassMetaData;
import org.jpox.metadata.ClassPersistenceModifier;
import org.jpox.metadata.ExtensionMetaData;
import org.jpox.metadata.IdentityStrategy;
import org.jpox.metadata.IdentityType;
import org.jpox.metadata.InheritanceMetaData;
import org.jpox.metadata.InheritanceStrategy;
import org.jpox.metadata.MapMetaData;
import org.jpox.metadata.SequenceMetaData;
import org.jpox.plugin.ConfigurationElement;
import org.jpox.resource.ManagedConnectionImpl;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.DatastoreContainerObject;
import org.jpox.store.DatastoreIdentifier;
import org.jpox.store.DatastoreObject;
import org.jpox.store.FetchStatement;
import org.jpox.store.IdentifierFactory;
import org.jpox.store.OID;
import org.jpox.store.SCOID;
import org.jpox.store.StoreData;
import org.jpox.store.StoreManager;
import org.jpox.store.exceptions.NoExtentException;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.poid.PoidConnectionProvider;
import org.jpox.store.poid.PoidGenerator;
import org.jpox.store.rdbms.adapter.RDBMSAdapter;
import org.jpox.store.rdbms.adapter.RDBMSAdapterFactory;
import org.jpox.store.rdbms.columninfo.ColumnInfo;
import org.jpox.store.rdbms.extent.ClassTableExtent;
import org.jpox.store.rdbms.extent.ClassViewExtent;
import org.jpox.store.rdbms.poid.AbstractRDBMSPoidGenerator;
import org.jpox.store.rdbms.request.DeleteRequest;
import org.jpox.store.rdbms.request.FetchRequest;
import org.jpox.store.rdbms.request.InsertRequest;
import org.jpox.store.rdbms.request.LocateRequest;
import org.jpox.store.rdbms.request.UpdateRequest;
import org.jpox.store.rdbms.scostore.FKArrayStore;
import org.jpox.store.rdbms.scostore.FKListStore;
import org.jpox.store.rdbms.scostore.FKMapStore;
import org.jpox.store.rdbms.scostore.FKSetStore;
import org.jpox.store.rdbms.scostore.JoinArrayStore;
import org.jpox.store.rdbms.scostore.JoinListStore;
import org.jpox.store.rdbms.scostore.JoinMapStore;
import org.jpox.store.rdbms.scostore.JoinSetStore;
import org.jpox.store.rdbms.sqlidentifier.SQLIdentifier;
import org.jpox.store.rdbms.table.AbstractTable;
import org.jpox.store.rdbms.table.ArrayTable;
import org.jpox.store.rdbms.table.ClassTable;
import org.jpox.store.rdbms.table.ClassView;
import org.jpox.store.rdbms.table.CollectionTable;
import org.jpox.store.rdbms.table.JoinTable;
import org.jpox.store.rdbms.table.ListTable;
import org.jpox.store.rdbms.table.MapTable;
import org.jpox.store.rdbms.table.ProbeTable;
import org.jpox.store.rdbms.table.SetTable;
import org.jpox.store.rdbms.table.Table;
import org.jpox.store.rdbms.table.TableImpl;
import org.jpox.store.rdbms.table.ViewImpl;
import org.jpox.store.scostore.ArrayStore;
import org.jpox.store.scostore.CollectionStore;
import org.jpox.store.scostore.MapStore;
import org.jpox.util.ClassUtils;
import org.jpox.util.FailoverUtils;
import org.jpox.util.JPOXLogger;
import org.jpox.util.JavaUtils;
import org.jpox.util.Localiser;
import org.jpox.util.MacroString;
import org.jpox.util.MultiMap;
import org.jpox.util.SoftValueMap;
import org.jpox.util.StringUtils;
import org.jpox.util.TypeConversionHelper;

/**
 * StoreManager for RDBMS datastores. Provided by the extension key "jdbc"
 * and accepts datastore URLs valid for JDBC.
 * <p>
 * The RDBMS manager's responsibilities extend those for StoreManager to add :
 * <ul>
 * <li>implements insert(), fetch(), update(), delete() in the interface to the StateManager.</li>
 * <li>Providing cached access to JDBC database metadata (in particular column information).</li>
 * <li>Resolving SQL identifier macros to actual SQL identifiers.</li>
 * </ul>
 *
 * @version $Revision: 1.217.2.5 $
 */
public class RDBMSManager extends StoreManager
{
    /** Localiser for messages. */
    protected static final Localiser LOCALISER_RDBMS = Localiser.getInstance("org.jpox.store.rdbms.Localisation");

    /** The amount of time before we expire any cached column info. Hardcoded to five minutes. */
    private static final int COLUMN_INFO_EXPIRATION_MS = 5 * 60 * 1000;

    /** Catalog name for the RDBMS database */
    private String catalogName = null;

    /** Schema name for the RDBMS database */
    private String schemaName = null;

    /** Map of ColumnInfo for tables, keyed by the fully-qualified table name. */
    private Map columnInfoByTableName = new HashMap();

    /** Map of ColumnInfo for column name, keyed by the fully-qualified column name. */
    private Map columnInfoByColumnName = new HashMap();
    
    /** Timestamp when the ColumnInfo was refreshed. */
    private long columnInfoReadTimestamp = -1;

    /** The cache of database requests. Access is synchronized on the map object itself. */
    private Map requestsByID = Collections.synchronizedMap(new SoftValueMap());

    /**
     * The active class adder transaction, if any. Some RDBMSManager methods are
     * called recursively in the course of adding new classes. This field allows
     * such methods to coordinate with the active ClassAdder transaction.
     * Recursive methods include:
     * <ul>
     * <li>addClasses()</li>
     * <li>addSetTable()</li>
     * <li>addMapTable()</li>
     * <li>addListTable()</li>
     * <li>addArrayTable()</li>
     * </ul>
     * Access is synchronized on the RDBMSManager itself. 
     * Invariant: classAdder == null if RDBMSManager is unlocked.
     */
    private ClassAdder classAdder = null;

    /** Object to be used for synchronising within the class addition process.*/
    private final Object CLASSADDER_MUTEX = new Object();

    /**
     * Constructs a new RDBMSManager. On successful return the new RDBMSManager
     * will have successfully connected to the database with the given
     * credentials and determined the schema name, but will not have inspected
     * the schema contents any further. The contents (tables, views, etc.) will
     * be subsequently created and/or validated on-demand as the application
     * accesses persistent classes.
     * <p>
     * To avoid creating unnecessary redundant RDBMSManagers, new RDBMSManagers
     * should always be obtained from the {@link org.jpox.store.StoreManagerFactory}, 
     * rather than constructed directly.
     * 
     * @param clr the ClassLoaderResolver
     * @param pmf The corresponding PersistenceManagerFactory. This factory's
     *            non-transactional data source will be used to get database
     *            connections as needed to perform management functions.
     * @param userName The database user name.
     * @param password The database user's password.
     * @exception JDODataStoreException
     *                If the database could not be accessed or the name of the
     *                schema could not be determined.
     * @see org.jpox.store.StoreManagerFactory
     */
    public RDBMSManager(ClassLoaderResolver clr, AbstractPersistenceManagerFactory pmf, String userName, String password)
    {
        super(clr, pmf, userName, password);

        // Retrieve the Database Adapter for this datastore
        try
        {
            Connection conn;
            if (userName == null)
            {
                conn = FailoverUtils.getConnection(ds, false);
            }
            else
            {
                conn = FailoverUtils.getConnection(ds, userName, password, false);
            }

            try
            {
                dba = RDBMSAdapterFactory.getInstance().getDatastoreAdapter(clr, conn, 
                    pmf.getPMFConfiguration().getDatastoreAdapterClassName(), pmf.getPMFContext().getPluginManager());
                dba.loadDatastoreMapping(pmf.getPMFContext().getPluginManager(), clr);

                // Check if the user has specified DefaultCatalogName/DefaultSchemaName and whether the DBA allows it
                if (pmf.getPMFConfiguration().getCatalog() != null && !((RDBMSAdapter)dba).supportsCatalogsInTableDefinitions())
                {
                   JPOXLogger.RDBMS.warn(LOCALISER_RDBMS.msg("RDBMS.Manager.DefaultCatalogNameUnusable",pmf.getPMFConfiguration().getCatalog()));
                }
                if (pmf.getPMFConfiguration().getSchema() != null && !((RDBMSAdapter)dba).supportsSchemasInTableDefinitions())
                {
                    JPOXLogger.RDBMS.warn(LOCALISER_RDBMS.msg("RDBMS.Manager.DefaultSchemaNameUnusable",pmf.getPMFConfiguration().getSchema()));
                }

                // Create an identifier factory - needs the database adapter to exist first
                initialiseIdentifierFactory(pmf);

                // Initialise the Schema "Name", and "Data"
                initialiseSchema(pmf.getPMFConfiguration().getAutoStartMechanism(), 
                                 pmf.getPMFConfiguration().getAutoStartMechanismMode(), 
                                 conn, clr);
            }
            finally
            {
                conn.close();
            }
        }
        catch (JDOException jdoex)
        {
            JPOXLogger.RDBMS_SCHEMA.error(LOCALISER_RDBMS.msg("RDBMS.Manager.DatabaseInitialisationError") + ' ' + LOCALISER_RDBMS.msg("RDBMS.Exception",jdoex));
            throw jdoex;
        }
        catch (Exception e1)
        {
            String msg=LOCALISER_RDBMS.msg("RDBMS.Manager.DatabaseInitialisationError") + ' ' + LOCALISER_RDBMS.msg("RDBMS.Manager.DatabaseInitialisationAdvice") + ' ' + LOCALISER_RDBMS.msg("RDBMS.Exception",e1);
            JPOXLogger.RDBMS_SCHEMA.error(msg);
            throw new JDODataStoreException(msg, e1);
        }
    }

    /**
     * Method to create the IdentifierFactory to be used by this store.
     * Relies on the datastore adapter existing before creation
     * @param pmf The PMF
     */
    protected void initialiseIdentifierFactory(AbstractPersistenceManagerFactory pmf)
    {
        String idFactoryName = pmf.getPMFConfiguration().getIdentifierFactory();
        String idFactoryClassName = pmfContext.getPluginManager().getAttributeValueForExtension("org.jpox.store_identifierfactory", 
            "name", idFactoryName, "class-name");
        if (idFactoryClassName == null)
        {
            throw new JDOFatalUserException(LOCALISER.msg("IdentifierFactory.FactoryNotFound", idFactoryName));
        }

        try
        {
            // Create the IdentifierFactory
            Class cls = Class.forName(idFactoryClassName);
            Class[] argTypes = new Class[] 
                {DatastoreAdapter.class, String.class, String.class, String.class, String.class, String.class, String.class};
            Object[] args = new Object[]
                {
                    dba, pmf.getPMFConfiguration().getCatalog(), pmf.getPMFConfiguration().getSchema(), 
                    pmf.getPMFConfiguration().getIdentifierCase(), pmf.getPMFConfiguration().getIdentifierWordSeparator(),
                    pmf.getPMFConfiguration().getIdentifierTablePrefix(), pmf.getPMFConfiguration().getIdentifierTableSuffix()
                };
            identifierFactory = (IdentifierFactory)ClassUtils.newInstance(cls, argTypes, args);
        }
        catch (ClassNotFoundException cnfe)
        {
            throw new JDOFatalUserException(LOCALISER.msg("IdentifierFactory.FactoryClassNotFound", idFactoryName, idFactoryClassName), cnfe);
        }
        catch (Exception e)
        {
            JPOXLogger.JDO.error(e);
            throw new JDOFatalInternalException(LOCALISER.msg("IdentifierFactory.FactoryError", idFactoryClassName), e);
        }

        if (JPOXLogger.JDO.isDebugEnabled())
        {
            JPOXLogger.JDO.debug(LOCALISER.msg("IdentifierFactory.Initialised", idFactoryName, identifierFactory.getNameOfIdentifierCase()));
        }
    }

    /**
     * Release of resources
     */
    public void close()
    {
        super.close();
        requestsByID.clear();
        columnInfoByTableName.clear();
        columnInfoByColumnName.clear();
        classAdder = null;
    }

    /**
     * Accessor for whether this query language is supported by this datastore.
     * @param language The language
     * @return Whether it is supported.
     * @see org.jpox.store.StoreManager#supportsQueryLanguage(java.lang.String)
     */
    public boolean supportsQueryLanguage(String language)
    {
        if (language == null)
        {
            return false;
        }

        // Find if datastore="jdbc" has an extension for name="{language}"
        String name = getPMFContext().getPluginManager().getAttributeValueForExtension("org.jpox.store_query_query", 
            "name", language, "datastore", "jdbc", "name");
        return (name != null);
    }

    /**
     * Method to return a datastore sequence for this datastore matching the passed sequence MetaData.
     * @param pm The PersistenceManager
     * @param seqmd SequenceMetaData
     * @return The Sequence
     */
    public Sequence getSequence(PersistenceManager pm, SequenceMetaData seqmd)
    {
        return new SequenceImpl(pm, this, seqmd);
    }

    /**
     * Method to obtain a nonmanaged transaction for the specified persistence manager for this store.
     * @param pm The PersistenceManager
     * @param userName The username
     * @param password The password
     * @return The transaction
     */
    public Transaction getNonmanagedTransaction(PersistenceManagerImpl pm, String userName, String password)
    {
        return new RDBMSNonmanagedTransaction(pm, userName, password);
    }

    /**
     * Method to obtain a nonmanaged transaction for the specified persistence manager for this store.
     * @param mc The ManagedConnection implementation
     * @param credential Password credential
     * @return The transaction
     */
    public Transaction getManagedTransaction(ManagedConnectionImpl mc, PasswordCredential credential)
    {
        return new RDBMSManagedTransaction(mc, credential);
    }

    /**
     * Accessor for a connection for the specified PersistenceManager, using its current transaction.
     * @param pm The PersistenceManager
     * @param forWriting Whether the connection is for writing
     * @param forQuery Whether the connection is for querying
     * @param useOpenedConnection Whetehr to use any opened connection
     * @return The Connection
     * @throws SQLException Thrown if an error occurs getting the connection
     */
    public Connection getConnection(PersistenceManager pm, boolean forWriting, boolean forQuery, boolean useOpenedConnection)
    throws SQLException
    {
        return ((RDBMSTransaction)pm.currentTransaction()).getConnection(forWriting, forQuery, useOpenedConnection);
    }

    /**
     * Accessor for a connection for the specified PersistenceManager, using its current transaction.
     * @param pm The PersistenceManager
     * @param forWriting Whether the connection is for writing
     * @param forQuery Whether the connection is for querying
     * @return The Connection
     * @throws SQLException Thrown if an error occurs getting the connection
     */
    public Connection getConnection(PersistenceManager pm, boolean forWriting, boolean forQuery)
    throws SQLException
    {
        return ((RDBMSTransaction)pm.currentTransaction()).getConnection(forWriting, forQuery);
    }

    /**
     * Method to return the Connection for this PersistenceManager for this datastore to the pool.
     * @param conn The Connection
     * @throws SQLException Thrown if an error occurs in the release of the connection.
     */
    public synchronized void releaseConnection(PersistenceManager pm, Connection conn)
    throws SQLException
    {
        ((RDBMSTransaction)pm.currentTransaction()).releaseConnection(conn);
    }

    /**
     * Utility to return a Connection not associated to the current transaction. 
     * Is simply a wrapper to the DatabaseAdapter getConnection method.
     * It will return a connection from a secondary DataSource
     * (e.g. javax.jdo.option.connectionFactory2Name),
     * if it is given. However, if a secondary DataSource is not configured,
     * the connection is obtained from the default DataSource.
     * @param isolation_level The transaction isolation scheme to use e.g
     * Connection.TRANSACTION_NONE
     * @return The Connection to the datastore
     * @throws SQLException
     */
    public Connection getConnection(int isolation_level) 
    throws SQLException
    {
        return ((RDBMSAdapter)dba).getConnection(dsNonTx, userName, password, isolation_level);
    }

    /**
     * Utility to close a Connection for the Datastore. Is simply a wrapper to
     * the DatabaseAdapter closeConnection method.
     * @param conn The Connection to the datastore
     * @throws SQLException
     */
    public void closeConnection(Connection conn)
    throws SQLException
    {
        ((RDBMSAdapter)dba).closeConnection(conn);
    }

    /**
     * Convenience method to create a new PreparedStatement.
     * Provided here so that we have a single place to apply control such
     * as timeouts to the statement before returning it.
     * TODO Could also be used as a way of applying cached PreparedStatements.
     * @param conn The Connection to use for the statement
     * @param stmtText Statement text
     * @param applyTimeout Whether this query will apply any timeout (where specified)
     * @return The PreparedStatement
     * @throws SQLException thrown if an error occurs creating the statement
     */
    public PreparedStatement getStatement(Connection conn, String stmtText, boolean applyTimeout)
    throws SQLException
    {
        return getStatement(conn, stmtText, applyTimeout, null, null);
    }

    /**
     * Convenience method to create a new PreparedStatement.
     * Provided here so that we have a single place to apply control such
     * as timeouts, result set types etc to the statement before returning it.
     * TODO Could also be used as a way of applying cached PreparedStatements.
     * @param conn The Connection to use for the statement
     * @param stmtText Statement text
     * @param applyTimeout Whether this query will apply any timeout (where specified)
     * @param resultSetType Type of result set
     * @param resultSetConcurrency Concurrency for the result set
     * @return The PreparedStatement
     * @throws SQLException thrown if an error occurs creating the statement
     */
    public PreparedStatement getStatement(Connection conn, String stmtText, boolean applyTimeout, String resultSetType, String resultSetConcurrency)
    throws SQLException
    {
        PreparedStatement ps = null;

        if (resultSetType != null || resultSetConcurrency != null)
        {
            int rsTypeValue = ResultSet.TYPE_FORWARD_ONLY;
            if (resultSetType.equals("scroll-sensitive"))
            {
                rsTypeValue = ResultSet.TYPE_SCROLL_SENSITIVE;
            }
            else if (resultSetType.equals("scroll-insensitive"))
            {
                rsTypeValue = ResultSet.TYPE_SCROLL_INSENSITIVE;
            }
            
            int rsConcurrencyValue = ResultSet.CONCUR_READ_ONLY;
            if (resultSetConcurrency.equals("updateable"))
            {
                rsConcurrencyValue = ResultSet.CONCUR_UPDATABLE;
            }

            ps = conn.prepareStatement(stmtText, rsTypeValue, rsConcurrencyValue);
        }
        else
        {
            ps = conn.prepareStatement(stmtText);
        }

        if (applyTimeout)
        {
            int queryTimeout = getPMFContext().getPmfConfiguration().getQueryTimeout();
            if (queryTimeout > 0)
            {
                // Apply any query timeout
                ps.setQueryTimeout(queryTimeout);
            }
        }
        
        if (JPOXLogger.RDBMS_SQL.isDebugEnabled() && JavaUtils.isJRE1_4OrAbove())
        {
            ps = new ParamLoggingPreparedStatement(ps, stmtText);
        }

        return ps;
    }

    /**
     * Method to execute a PreparedStatement update.
     * Prints logging information about timings.
     * @param stmt The statement text
     * @param ps The Prepared Statement
     * @return The numer of rows affected (as per PreparedStatement.executeUpdate)
     * @throws SQLException Thrown if an error occurs
     */
    public int executeStatementUpdate(String stmt, PreparedStatement ps)
    throws SQLException
    {
        if (JPOXLogger.RDBMS_SQL.isDebugEnabled())
        {
            if (ps instanceof ParamLoggingPreparedStatement)
            {
                JPOXLogger.RDBMS_SQL.debug(((ParamLoggingPreparedStatement)ps).getStatementWithParamsReplaced());
            }
            else
            {
                JPOXLogger.RDBMS_SQL.debug(stmt);
            }
        }

        long startTime = System.currentTimeMillis();
        int ind = ps.executeUpdate();

        if (JPOXLogger.RDBMS_SQL.isDebugEnabled())
        {
            JPOXLogger.RDBMS_SQL.debug(LOCALISER_RDBMS.msg("JDBC.Statement.ExecutionTime",(System.currentTimeMillis() - startTime)));
        }

        return ind;
    }

    /**
     * Method to execute a PreparedStatement batch update.
     * Prints logging information about timings.
     * @param stmt The statement text
     * @param ps The Prepared Statement
     * @return The array of numer of rows affected (as per PreparedStatement.executeBatch)
     * @throws SQLException Thrown if an error occurs
     */
    public int[] executeStatementBatch(String stmt, PreparedStatement ps)
    throws SQLException
    {
        if (JPOXLogger.RDBMS_SQL.isDebugEnabled())
        {
            if (ps instanceof ParamLoggingPreparedStatement)
            {
                JPOXLogger.RDBMS_SQL.debug(((ParamLoggingPreparedStatement)ps).getStatementWithParamsReplaced());
            }
            else
            {
                JPOXLogger.RDBMS_SQL.debug(stmt);
            }
        }

        long startTime = System.currentTimeMillis();
        int[] ind = ps.executeBatch();

        if (JPOXLogger.RDBMS_SQL.isDebugEnabled())
        {
            JPOXLogger.RDBMS_SQL.debug(LOCALISER_RDBMS.msg("JDBC.Statement.ExecutionTime",(System.currentTimeMillis() - startTime)));
        }

        return ind;
    }

    /**
     * Method to execute a PreparedStatement query, and return the ResultSet.
     * Prints logging information about timings.
     * @param stmt The statement text
     * @param ps The Prepared Statement
     * @return The ResultSet from the query
     * @throws SQLException Thrown if an error occurs
     */
    public ResultSet executeStatementQuery(String stmt, PreparedStatement ps)
    throws SQLException
    {
        if (JPOXLogger.RDBMS_SQL.isDebugEnabled())
        {
            if (ps instanceof ParamLoggingPreparedStatement)
            {
                JPOXLogger.RDBMS_SQL.debug(((ParamLoggingPreparedStatement)ps).getStatementWithParamsReplaced());
            }
            else
            {
                JPOXLogger.RDBMS_SQL.debug(stmt);
            }
        }

        long startTime = System.currentTimeMillis();
        ResultSet rs = ps.executeQuery();

        if (JPOXLogger.RDBMS_SQL.isDebugEnabled())
        {
            JPOXLogger.RDBMS_SQL.debug(LOCALISER_RDBMS.msg("JDBC.Statement.ExecutionTime", (System.currentTimeMillis() - startTime)));
        }

        return rs;
    }

    /**
     * Initialises the schema name for the datastore, and (optionally) the
     * schema table (and associated initial schema data).
     * @param auto_start_mechanism Name of AutoStart mechanism
     * @param auto_start_mechanism_mode Mode of AutoStart mechanism
     * @param conn A connection to the database
     */
    private void initialiseSchema(String auto_start_mechanism,
                                  String auto_start_mechanism_mode,
                                  Connection conn,
                                  ClassLoaderResolver clr)
    throws Exception
    {
        // Initialise the Catalog/Schema names
        RDBMSAdapter rdba = (RDBMSAdapter)dba;
        if ((pmfContext.getPmfConfiguration().getCatalog() != null && rdba.supportsCatalogsInTableDefinitions()) ||
            (pmfContext.getPmfConfiguration().getSchema() != null && rdba.supportsSchemasInTableDefinitions()))
        {
            // User-specified catalog/schema
            catalogName = pmfContext.getPmfConfiguration().getCatalog();
            schemaName = pmfContext.getPmfConfiguration().getSchema();
        }
        else
        {
            // TODO Should we bother with this if the RDBMS doesnt support catalog/schema in the table identifiers ?
            // Try to find the catalog/schema from the connection
            try
            {
                try
                {
                    catalogName = rdba.getCatalogName(conn);
                    schemaName = rdba.getSchemaName(conn);
                }
                catch (UnsupportedOperationException e)
                {
                    if (!readOnlyDatastore && !fixedDatastore)
                    {
                        // If we aren't a read-only datastore, try to create a table and then 
                        // retrieve its details, so as to obtain the catalog, schema. 
                        ProbeTable pt = new ProbeTable(this);
                        pt.initialize(clr);
                        pt.create(conn);
                        try
                        {
                            String[] schema_details = pt.findSchemaDetails(conn);
                            if (schema_details != null)
                            {
                                catalogName = schema_details[0];
                                schemaName = schema_details[1];
                            }
                        }
                        finally
                        {
                            pt.drop(conn);
                        }
                    }
                }
            }
            catch (SQLException e)
            {
                String msg = LOCALISER_RDBMS.msg("RDBMS.Manager.DatabaseConnectionError") + ' ' + 
                    LOCALISER_RDBMS.msg("RDBMS.Manager.DatabaseInitialisationAdvice") + ' ' + e.toString();
                JPOXLogger.RDBMS_SCHEMA.warn(msg);
                // This is only logged as a warning since if the JDBC driver has some issue creating the ProbeTable we would be stuck
                // We need to allow SchemaTool "dbinfo" mode to work in all circumstances.
            }
        }

        if ((readOnlyDatastore || fixedDatastore) && auto_start_mechanism.equals("SchemaTable"))
        {
            auto_start_mechanism = "None";
        }

        if (JPOXLogger.RDBMS_SCHEMA.isInfoEnabled())
        {
            JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.SchemaDataInitialise", catalogName, schemaName, 
                auto_start_mechanism));
        }

        if (!readOnlyDatastore)
        {
            dba.initialiseDatastore(conn);
        }
        
        // AutoStarter - Load up any startup class names
        initialiseAutoStart(auto_start_mechanism, auto_start_mechanism_mode, clr);

        if (JPOXLogger.RDBMS_SCHEMA.isInfoEnabled())
        {
            if (readOnlyDatastore)
            {
                JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.SchemaDataInitialisedReadOnly",
                    catalogName, schemaName, "" + storeDataByClass.size()));
            }
            else if (fixedDatastore)
            {
                JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.SchemaDataInitialisedFixed",
                        catalogName, schemaName, "" + storeDataByClass.size()));
            }
            else
            {
                JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.SchemaDataInitialised",
                        catalogName, schemaName, "" + storeDataByClass.size()));
            }
        }
    }

    /**
     * Clears all knowledge of tables, cached requests, metadata, etc and resets
     * the store manager to its initial state.
     */
    private void clearSchemaData()
    {
        deregisterAllStoreData();

        columnInfoByTableName.clear();
        columnInfoReadTimestamp = -1;
        requestsByID.clear();
    }

    /**
     * Accessor for the (default) RDBMS catalog name.
     * 
     * @return The catalog name.
     */
    public String getCatalogName()
    {
        return catalogName;
    }

    /**
     * Accessor for the (default) RDBMS schema name.
     * 
     * @return The schema name.
     */
    public String getSchemaName()
    {
        return schemaName;
    }

    // ----------------------------- Class Management -------------------------------

    /**
     * Method to add several (PersistenceCapable) classes to the store manager's
     * set of supported classes. This will create any necessary database objects
     * (tables, views, constraints, indexes etc). This will cause the addition
     * of any related classes.
     * 
     * @param classNames Name of the class(es) to be added.
     * @param clr The ClassLoaderResolver
     * @param writer Optional writer when you just want the DDL for persisting the specified classes
     */
    public void addClasses(String[] classNames, ClassLoaderResolver clr, FileWriter writer)
    {
        if (writer != null)
        {
            AbstractTable.setDdlDumpWriter(writer);
        }

        synchronized (CLASSADDER_MUTEX)
        {
            if (classAdder != null)
            {
                // addClasses() has been recursively re-entered: just add table
                // objects for the requested classes and return.
                classAdder.addClasses(classNames, clr);
                return;
            }
        }
        if (classNames != null && classNames.length > 0)
        {
            new ClassAdder(classNames, writer).execute(clr);
        }
    }

    /**
     * Method to add a (PersistenceCapable) class to the store manager's set of
     * supported classes. This will create any necessary database objects. This
     * will cause the addition of any related classes.
     * 
     * @param className Name of the class to be added.
     * @param clr The ClassLoaderResolver
     */
    public void addClass(String className, ClassLoaderResolver clr)
    {
        addClasses(new String[] {className}, clr, null);
    }

    /**
     * Utility to remove all classes that we are managing.
     * @param clr The ClassLoaderResolver
     */
    public void removeAllClasses(ClassLoaderResolver clr)
    {
        MgmtTransaction mtx = new MgmtTransaction(Connection.TRANSACTION_READ_COMMITTED)
        {
            public String toString()
            {
                return LOCALISER_RDBMS.msg("RDBMS.Manager.RemoveAllClassesFromSchema", catalogName, schemaName);
            }

            protected void run(ClassLoaderResolver clr)
            throws SQLException
            {
                synchronized (RDBMSManager.this)
                {
                    boolean success = true;
                    try
                    {
                        JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.RemoveAllClassesFromSchema", 
                            catalogName, schemaName));

                        Map baseTablesByName = new HashMap();
                        Map viewsByName = new HashMap();
                        for (Iterator i = storeDataByClass.values().iterator(); i.hasNext();)
                        {
                            RDBMSStoreData data = (RDBMSStoreData) i.next();
                            if (JPOXLogger.RDBMS_SCHEMA.isInfoEnabled())
                            {
                                JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.RemoveClass",data.getName()));
                            }
                            
                            // If the class has a table/view to remove, add it to the list
                            if (data.hasTable())
                            {
                                if (data.mapsToView())
                                {
                                    viewsByName.put(data.getDatastoreIdentifier(), 
                                        data.getDatastoreContainerObject());
                                }
                                else
                                {
                                    baseTablesByName.put(data.getDatastoreIdentifier(),
                                        data.getDatastoreContainerObject());
                                }
                            }
                        }
                        
                        // Remove tables, table constraints and views in
                        // reverse order from which they were created
                        Iterator viewsIter = viewsByName.values().iterator();
                        while (viewsIter.hasNext())
                        {
                            ((ViewImpl) viewsIter.next()).drop(getCurrentConnection());
                        }

                        Iterator tablesIter = baseTablesByName.values().iterator();
                        while (tablesIter.hasNext())
                        {
                            ((TableImpl) tablesIter.next()).dropConstraints(getCurrentConnection());
                        }
                        tablesIter = baseTablesByName.values().iterator();
                        while (tablesIter.hasNext())
                        {
                            ((TableImpl) tablesIter.next()).drop(getCurrentConnection());
                        }
                    }
                    catch (Exception e)
                    {
                        success = false;
                        String errorMsg = LOCALISER_RDBMS.msg("RDBMS.Manager.RemoveClassesExceptionError", e);
                        JPOXLogger.RDBMS_SCHEMA.error(errorMsg);
                        throw new JDOUserException(errorMsg, e);
                    }
                    finally
                    {
                        // Only empty our schema data if we succeeded in
                        // dropping everything with no error.
                        if (success)
                        {
                            clearSchemaData();
                        }
                    }
                }
            }
        };
        mtx.execute(clr);
    }

    // ---------------------------------------------------------------------------------------

    /**
     * Method to create a new fetch statement for the passed table.
     * @param table The table to fetch from
     * @return The fetch statement
     */
    public FetchStatement getFetchStatement(DatastoreContainerObject table)
    {
        return new RDBMSFetchStatement(table);
    }

    /**
     * Called by (container) Mapping objects to request the creation of a DatastoreObject (table).
     * If the specified field doesn't require a join table then this returns null.
     * If the join table already exists, then this returns it.
     * @param fmd The field metadata describing the field.
     * @param clr The ClassLoaderResolver
     * @return The container object (SetTable/ListTable/MapTable/ArrayTable)
     */
    public synchronized DatastoreContainerObject newJoinDatastoreContainerObject(AbstractPropertyMetaData fmd, ClassLoaderResolver clr)
    {
        if (fmd.getJoinMetaData() == null)
        {
            AbstractPropertyMetaData relatedFmd = fmd.getRelatedFieldMetaData(clr);
            if (relatedFmd != null && relatedFmd.getJoinMetaData() != null)
            {
                // Join specified at other end of a bidirectional relation so create a join table
            }
            else
            {
                Class element_class;
                if (fmd.hasCollection())
                {
                    element_class = clr.classForName(fmd.getCollection().getElementType());
                }
                else if (fmd.hasMap())
                {
                    MapMetaData mmd = (MapMetaData)fmd.getContainer();
                    if (fmd.getValueMetaData() != null && fmd.getValueMetaData().getMappedBy() != null)
                    {
                        // value stored in the key table
                        element_class = clr.classForName(mmd.getKeyType());
                    }
                    else if (fmd.getKeyMetaData() != null && fmd.getKeyMetaData().getMappedBy() != null)
                    {
                        // key stored in the value table
                        element_class = clr.classForName(mmd.getValueType());
                    }
                    else
                    {
                        // No information given for what is stored in what, so throw it back to the user to fix the input :-)
                        throw new JDOUserException(LOCALISER_RDBMS.msg("RDBMS.Manager.MapWithoutJoinWithoutMappedBy", fmd.getFullFieldName()));
                    }
                }
                else if (fmd.hasArray())
                {
                    element_class = clr.classForName(fmd.getTypeName()).getComponentType();
                }
                else
                {
                    // what is this? should not happen
                    return null;
                }

                // Check that the element class has MetaData
                if (getMetaDataManager().getMetaDataForClass(element_class, clr) != null)
                {
                    // FK relationship, so no join table
                    return null;
                }
                else if (getPMFContext().getTypeManager().isReferenceType(element_class))
                {
                    // reference type using FK relationship so no join table
                    return null;
                }

                // Trap all non-PC elements that haven't had a join table specified but need one
                throw new JDOUserException(LOCALISER_RDBMS.msg("RDBMS.Manager.NonPersistenceCapableCollectionElementNeedsJoin", 
                    fmd.getFullFieldName(), fmd.toString()));
            }
        }

        // Check if the join table already exists
        DatastoreContainerObject joinTable = getDatastoreContainerObject(fmd);
        if (joinTable != null)
        {
            return joinTable;
        }

        // Create a new join table for the container
        if (classAdder == null)
        {
            throw new IllegalStateException(LOCALISER_RDBMS.msg("RDBMS.Manager.JoinTableCreationError"));
        }

        if (fmd.getType().isArray())
        {
            // Use Array table for array types
            return classAdder.addJoinTableForContainer(fmd, clr, ClassAdder.JOIN_TABLE_ARRAY);
        }
        else if (getPMFContext().getTypeManager().isSCOList(fmd.getType()))
        {
            // Use List join table for supported list types
            return classAdder.addJoinTableForContainer(fmd, clr, ClassAdder.JOIN_TABLE_LIST);
        }
        else if (getPMFContext().getTypeManager().isSCOMap(fmd.getType()))
        {
            // Use Map join table for supported map types
            return classAdder.addJoinTableForContainer(fmd, clr, ClassAdder.JOIN_TABLE_MAP);
        }
        else
        {
            // Use Set join table for collection/set types
            return classAdder.addJoinTableForContainer(fmd, clr, ClassAdder.JOIN_TABLE_SET);
        }
    }

    /**
     * Method to return a backing store for a Collection, consistent with this store and the instantiated type.
     * @param fmd MetaData for the field that has this collection
     * @param datastoreTable The datastore object
     * @param clr ClassLoader resolver
     * @param instantiated Whether it is already instantiated
     * @param listBased whether the collection is list based
     * @return The backing store of this collection in this store
     */
    public CollectionStore getBackingStoreForCollection(AbstractPropertyMetaData fmd, DatastoreObject datastoreTable, 
            ClassLoaderResolver clr, boolean instantiated, boolean listBased)
    {
        CollectionStore store = null;
        if (!instantiated)
        {
            // No type to base it on so create it based on the field declared type
            if (datastoreTable == null)
            {
                // We need an "Inverse". Where the owner column resides in the element table
                if (getPMFContext().getTypeManager().isSCOList(fmd.getType()))
                {
                    store = new FKListStore(fmd, this, clr);
                }
                else
                {
                    store = new FKSetStore(fmd, this, clr);
                }
            }
            else
            {
                // If a join table is present we need a "Normal"
                if (getPMFContext().getTypeManager().isSCOList(fmd.getType()))
                {
                    store = new JoinListStore(fmd, (CollectionTable)datastoreTable, clr);
                }
                else
                {
                    store = new JoinSetStore(fmd, (CollectionTable)datastoreTable, clr);
                }
            }
        }
        else
        {
            // Instantiated type specified so use it to pick the associated backing store
            if (datastoreTable == null)
            {
                if (listBased)
                {
                    // List required
                    store = new FKListStore(fmd, this, clr);
                }
                else
                {
                    // Set required
                    store = new FKSetStore(fmd, this, clr);
                }
            }
            else
            {
                if (listBased)
                {
                    // List required
                    store = new JoinListStore(fmd, (CollectionTable)datastoreTable, clr);
                }
                else
                {
                    // Set required
                    store = new JoinSetStore(fmd, (CollectionTable)datastoreTable, clr);
                }
            }
        }
        return store;
    }

    /**
     * Method to return a backing store for a Map, consistent with this store and the instantiated type.
     * @param fmd MetaData for the field that has this map
     * @param datastoreTable The datastore object
     * @param clr ClassLoader resolver
     * @return The backing store of this map in this store
     */
    public MapStore getBackingStoreForMap(AbstractPropertyMetaData fmd, DatastoreObject datastoreTable, ClassLoaderResolver clr)
    {
        MapStore store = null;
        if (datastoreTable == null)
        {
            store = new FKMapStore(fmd, this, clr);
        }
        else
        {
            store = new JoinMapStore((MapTable)datastoreTable, clr);
        }
        return store;
    }

    /**
     * Method to return a backing store for an array, consistent with this store and the instantiated type.
     * @param fmd MetaData for the field that has this array
     * @param datastoreTable The datastore object
     * @param clr ClassLoader resolver
     * @return The backing store of this array in this store
     */
    public ArrayStore getBackingStoreForArray(AbstractPropertyMetaData fmd, DatastoreObject datastoreTable, ClassLoaderResolver clr)
    {
        ArrayStore store = null;
        if (datastoreTable != null)
        {
            store = new JoinArrayStore((ArrayTable)datastoreTable, clr);
        }
        else
        {
            store = new FKArrayStore(fmd, this, clr);
        }
        return store;
    }

    /**
     * Returns the class corresponding to the given object JDO ID. If the object
     * is an OID, return the PersistenceCapable class. If the object is a SCOID,
     * return the SCO class. If the object is an AppID PK, return the PersistenceCapable
     * class that the id relates to. If SingleFieldIdentity return the associated PC class.
     * If the object id is an application id and the user supplies the "pm" argument then
     * a check can be performed in the datastore where necessary.
     * @param id The JDO identity of some object.
     * @param clr ClassLoader resolver
     * @param pm Persistence Manager (optional - to allow check in the datastore)
     * @return For datastore identity, return the class of the corresponding
     * object. For application identity, return the class of the corresponding
     * object or null if object does not exist.
     * @exception ClassCastException If the type of ID is not recognized (
     * {@link OID}or {@link SCOID}).
     */
    public String getClassNameForObjectID(Object id, ClassLoaderResolver clr, PersistenceManager pm)
    {
        // Object is a SCOID
        if (id instanceof SCOID)
        {
            return ((SCOID) id).getSCOClass();
        }
        // Multiple subclasses so we need to find which one using a query
        // Only include classes which have a table.
        List classTree = new ArrayList();

        String targetClassName = null;
        AbstractClassMetaData cmd = null;
        // Object is an OID
        if (id instanceof OID)
        {
        	OID oid = (OID) id;
        	cmd = getMetaDataManager().getMetaDataForClass(oid.getPcClass(), clr);
        	if( cmd.getDiscriminatorStrategy() == null)
        	{
	            // Using SingleFieldIdentity so can assume that object is of the target class or a subclass
	            List subclasses = new ArrayList(this.getSubClassesForClass(oid.getPcClass(), true, clr));
	            if (subclasses.size() < 1)
	            {
	            	JPOXLogger.JDO.debug("1) Id \""+id+"\" has been determined to be the id of class "+oid.getPcClass());
	                // No subclasses so must be an instance of the OID class
	                return oid.getPcClass();
	            }
	            targetClassName = oid.getPcClass();
        	}
        }
        else if (id instanceof SingleFieldIdentity)
        {
            // Using SingleFieldIdentity so can assume that object is of the target class or a subclass
            targetClassName = ((SingleFieldIdentity)id).getTargetClassName();

            cmd = getMetaDataManager().getMetaDataForClass(targetClassName, clr);
            if (cmd.getIdentityType() != IdentityType.APPLICATION || !cmd.getObjectidClass().equals(id.getClass().getName()))
            {
                throw new JDOUserException(LOCALISER_RDBMS.msg("RDBMS.Manager.IDAssignedToIncorrectClass", id, cmd.getFullClassName()));
            }
        	if( cmd.getDiscriminatorStrategy() != null)
        	{    
        		targetClassName = null;
        	}
        }
        else
        {
        	// Find all of the application identity PK classes of this type
        	Collection c = (Collection) storeDataByAppIdClass.get(id.getClass().getName());
        	if (c != null)
        	{
        		Iterator iter = c.iterator();
        		while (iter.hasNext())
        		{
        			RDBMSStoreData store_data = (RDBMSStoreData)iter.next();
        			if (store_data.hasTable())
        			{
        				classTree.add(store_data);
        			}
        		}
        	}
        }

        //OID or SingleFieldIdentity
        if (targetClassName != null)
        {
            RDBMSStoreData data = (RDBMSStoreData)storeDataByClass.get(targetClassName);
            if (data != null)
            {
                if (data.hasTable())
                {
                    classTree.add(data);
                }
                HashSet subclasses = getSubClassesForClass(targetClassName, true, clr);
                Iterator subclassesIter = subclasses.iterator();
                while (subclassesIter.hasNext())
                {
                    String subclassName = (String)subclassesIter.next();
                    RDBMSStoreData subclassData = (RDBMSStoreData)storeDataByClass.get(subclassName);
                    if (subclassData.hasTable())
                    {
                        classTree.add(subclassData);
                    }
                }
            }
        }

        if (classTree.size() == 0 && (cmd == null || cmd.getDiscriminatorStrategy()==null))
        {
        	JPOXLogger.JDO.debug("2) Id \""+id+"\" has not determined to the class");
            return null;
        }
        else if (classTree.size() == 1)
        {
            cmd = getMetaDataManager().getMetaDataForClass(((RDBMSStoreData)classTree.get(0)).getName(), clr);
            if(cmd == null || cmd.getDiscriminatorStrategy()==null)
            {
            	JPOXLogger.JDO.debug("3) Id \""+id+"\" has been determined to be the id of class "+((RDBMSStoreData)classTree.get(0)).getName());
            	return ((RDBMSStoreData)classTree.get(0)).getName();
            }
        }
        if (pm != null)
        {
        	// Perform a check on the exact object inheritance level with this key (uses SQL query)
        	String className = null;
        	if( cmd != null && cmd.getDiscriminatorStrategy()!=null)
        	{
        		DatastoreClass primaryTable = pm.getStoreManager().getDatastoreClass(cmd.getFullClassName(), pm.getClassLoaderResolver());
        		if (primaryTable != null)
        		{
        			classTree.add(storeDataByClass.get(cmd.getFullClassName()));
        			HashSet subclasses = getSubClassesForClass(cmd.getFullClassName(), true, clr);
        			Iterator subclassesIter = subclasses.iterator();
        			while (subclassesIter.hasNext())
        			{
        				String subclassName = (String)subclassesIter.next();
        				RDBMSStoreData subclassData = (RDBMSStoreData)storeDataByClass.get(subclassName);
        				if (subclassData.hasTable())
        				{
        					classTree.add(subclassData);
        				}
        			}
        			className = RDBMSStoreHelper.getClassNameForIdKeyUsingDiscriminator(pm, this, id, classTree);
        			if (className != null)
        			{
                    	JPOXLogger.JDO.debug("4) Id \""+id+"\" has been determined to be the id of class "+className);
        				return className;
        			}
        			return null;
        		}
        	}
        	else
        	{
        		className = RDBMSStoreHelper.getClassNameForIdKeyUsingUnion(pm, this, id, classTree);
        	}
        	if (className != null)
        	{
            	JPOXLogger.JDO.debug("6) Id \""+id+"\" has been determined to be the id of class "+className);
        		return className;
        	}

        	if (id instanceof OID)
        	{
            	JPOXLogger.JDO.debug("7) Id \""+id+"\" has been determined to be the id of class "+((OID)id).getPcClass());
        		return ((OID)id).getPcClass();
        	}
        }
        else
        {
        	// Check not possible so just return the first
        	JPOXLogger.JDO.debug("8) Id \""+id+"\" has been determined to be the id of class "+((RDBMSStoreData)classTree.get(0)).getName());
        	return ((RDBMSStoreData)classTree.get(0)).getName();
        }

        return null;
    }

    /**
     * Accessor for an Extent for a class, and its subclasses.
     * @param pm The Persistence Manager
     * @param c The (candidate) class to use for the Extent
     * @param subclasses Whether to include subclasses of 'c'
     * @return The Extent
     * @exception NoExtentException if an extent is not managed for the
     * specified class
     */
    public Extent getExtent(PersistenceManager pm, Class c, boolean subclasses)
    {
        AbstractClassMetaData cmd = getMetaDataManager().getMetaDataForClass(c, pm.getClassLoaderResolver());
        if (!cmd.isRequiresExtent())
        {
            throw new NoExtentException(c.getName());
        }

        DatastoreClass t = getDatastoreClass(cmd.getFullClassName(), pm.getClassLoaderResolver());
        if (t instanceof ClassView)
        {
            return new ClassViewExtent(pm, t, c, subclasses);
        }
        else if (t instanceof ClassTable)
        {
            return new ClassTableExtent(pm, t, c, subclasses);
        }
        else
        {
            // No specific table, so either has no table, or multiple
            // This happens with the "subclass-table" inheritance strategy
            AbstractClassMetaData[] cmds = getClassesManagingTableForClass(cmd, pm.getClassLoaderResolver());
            if (cmds != null)
            {
                DatastoreClass[] tables = new DatastoreClass[cmds.length];
                for (int i=0;i<cmds.length;i++)
                {
                    tables[i] = getDatastoreClass(cmds[i].getFullClassName(), pm.getClassLoaderResolver());
                }
                return new ClassTableExtent(pm, tables, c, subclasses);
            }
            else
            {
                throw new JDOUserException("Attempt to create an Extent for class " + c.getName() + " which has MetaData, yet no table was found! This should be impossible");
            }
        }
    }

    /**
     * Method to return the value for the specified strategy.
     * Supports the non-datastore strategies of JDO2. That is, it does not
     * support "identity" which is done elsewhere.
     * This method is synchronised so that several threads dont come in and update
     * the PoidManager connection details while another thread is allocating the POID
     * and change the connection that it should release after its work!
     * @param pm the PersistenceManager requesting an ID for the object
     * @param table The datastore table
     * @param cmd AbstractClassMetaData for the class containing this field
     * @param absoluteFieldNumber Absolute number of the field (-1 means datastore identity field)
     * @param strategy The strategy to use for defining the value
     * @param sequence Name of a sequence if the strategy is sequence
     * @param extensions Any extensions to control the value generation process.
     *                   For value-strategy tags this is in the field extension.
     *                   For datastore-identity tags this is in the datastore-identity extension.
     * @return The value for this strategy
     * @exception JDODataStoreException Thrown if an error occurs getting the value
     */
    public Object getStrategyValue(final PersistenceManager pm,
    		final DatastoreClass table,
            final AbstractClassMetaData cmd,
            final int absoluteFieldNumber,
            final IdentityStrategy strategy,
            final String sequence,
            final ExtensionMetaData[] extensions)
    {
        synchronized (this) // TODO Why is this synchronised? storeDataByClass has nothing needing synchronising
        {
            int size = storeDataByClass.size();
            if (size == 0)
            {
                String msg = LOCALISER_RDBMS.msg("RDBMS.Manager.SchemaDataUninitialisedForSchema",catalogName, schemaName);
                JPOXLogger.RDBMS_SCHEMA.error(msg);
                throw new JDOFatalInternalException(msg);
            }
        }

        if (strategy == null || strategy == IdentityStrategy.IDENTITY)
        {
            // "identity" is attributed by the datastore, and not here
            throw new JDOFatalInternalException("RDBMSManager.getStrategyValue called for invalid strategy \"" + strategy + "\"");
        }

        // POID Generators are cached against a name. Some POID Generators are unique per datastore.
        // Others are per class/field. The name that they are cached under relates to what they use.
        // Generate the name for the strategy for this field/class.
        String generatorNameKeyInManager = null;
        String poidGeneratorClassName = null;
        ConfigurationElement elem = pmfContext.getPluginManager().getConfigurationElementForExtension("org.jpox.store_valuegenerator", 
            "name", strategy.toString());
        if (elem != null)
        {
            // Set the generator name (for use by the PoidManager)
            poidGeneratorClassName = elem.getAttribute("class-name");
            if (elem.getAttribute("unique") != null &&
                elem.getAttribute("unique").equalsIgnoreCase("true"))
            {
                // Unique value generator so set the key in PoidManager to the value generator name itself
                generatorNameKeyInManager = poidGeneratorClassName;
            }
        }

        if (generatorNameKeyInManager == null)
        {
            // Value generator is not unique so set based on class/field
            if (absoluteFieldNumber >= 0)
            {
                // Require generation for a field so use field name for the generator symbolic name
                generatorNameKeyInManager = cmd.getManagedFieldAbsolute(absoluteFieldNumber).getFullFieldName();
            }
            else
            {
                // Require generation for a datastore id field so use the class name for the generator symbolic name
                generatorNameKeyInManager = cmd.getFullClassName();
            }
        }

        // Try to find the generator from the PoidManager if we already have it managed
        PoidGenerator generator = poidManager.getPoidGenerator(generatorNameKeyInManager);
        if (generator == null)
        {
            // No PoidGenerator for that key found, so create one

            // Get base table with the required field
            // TODO The passed-in table may already be the right one - do this better
            DatastoreClass tbl = getDatastoreClass(cmd.getBaseAbstractClassMetaData().getFullClassName(), pm.getClassLoaderResolver());
            if (tbl == null)
            {
                tbl = table;
            }
            JavaTypeMapping m = null;
            if (absoluteFieldNumber >= 0)
            {
                m = tbl.getFieldMapping(cmd.getManagedFieldAbsolute(absoluteFieldNumber));
                if (m == null)
                {
                    // Field not mapped in root table so use passed-in table
                    m = table.getFieldMapping(cmd.getManagedFieldAbsolute(absoluteFieldNumber));
                    tbl = table;
                }
            }
            else
            {
                m = tbl.getIDMapping();
            }
            StringBuffer columnsName = new StringBuffer();
            for (int i = 0; i < m.getNumberOfDatastoreFields(); i++)
            {
                if (i > 0)
                {
                    columnsName.append(",");
                }
                columnsName.append(m.getDataStoreMapping(i).getDatastoreField().getIdentifier().toString());
            }

            // Set up the default properties available for all value generators
            Properties properties = new Properties();
            properties.setProperty("class-name", cmd.getFullClassName());
            properties.put("root-class-name", cmd.getBaseAbstractClassMetaData().getFullClassName());
            if (cmd.getManagedFieldAbsolute(absoluteFieldNumber) != null)
            {
                properties.setProperty("field-name", cmd.getManagedFieldAbsolute(absoluteFieldNumber).getFullFieldName());
            }
            if (cmd.getCatalog() != null)
            {
                properties.setProperty("catalog-name", cmd.getCatalog());
            }
            if (cmd.getSchema() != null)
            {
                properties.setProperty("schema-name", cmd.getSchema());
            }
            properties.setProperty("table-name", tbl.getIdentifier().toString());
            properties.setProperty("column-name", columnsName.toString());
            if (sequence != null)
            {
                properties.setProperty("sequence-name", sequence);
            }

            // Cater for NATIVE (pick the most suitable)
            if (strategy == IdentityStrategy.NATIVE)
            {
                // TODO This currently uses hardcoded (though JPOX-provided) value generator class names. Improve this.
                if (((RDBMSAdapter)dba).supportsSequences() && sequence != null)
                {
                    poidGeneratorClassName = "org.jpox.store.rdbms.poid.SequencePoidGenerator";
                }
                else
                {
                    poidGeneratorClassName = "org.jpox.store.rdbms.poid.SequenceTablePoidGenerator";
                }
            }

            // Add any id extension properties to the value generator properties
            if (extensions != null)
            {
                for (int i=0;i<extensions.length;i++)
                {
                    properties.put(extensions[i].getKey(), extensions[i].getValue());
                }
            }

            // Create the new PoidGenerator since the first time required (registers it with the PoidManager too)
            generator = poidManager.createPoidGenerator(pm.getClassLoaderResolver(), generatorNameKeyInManager,
                poidGeneratorClassName, properties, this, null);
        }

        // Get the next value from the generator
        Object oid = null;
        synchronized (generator)
        {
        	// Get the next value for this generator for this PersistenceManager
        	// Note : this is synchronised since we dont want to risk handing out this generator
        	// while its connectionProvider is set to that of a different PersistenceManager
        	// It maybe would be good to change PoidGenerator to have a next taking the connectionProvider
        	if (generator instanceof AbstractRDBMSPoidGenerator)
        	{
        		// RDBMS-based generator so set the connection provider
        		final RDBMSManager thisStoreMgr = this;
        		PoidConnectionProvider connProvider = new PoidConnectionProvider()
        		{
        			Connection conn;
        			public Object retrieveConnection()
        			{
        				try
        				{
        					if (pm.getPMFContext().getPmfConfiguration().getPoidTransactionAttribute().equalsIgnoreCase("UsePM"))
        					{
        						this.conn = thisStoreMgr.getConnection(pm, false, false);
        					}
        					else
        					{
        						this.conn = thisStoreMgr.getConnection(pmfContext.getPmfConfiguration().getPoidTransactionIsolationLevel());
        					}
        				}
        				catch (SQLException e)
        				{
        					String msg = LOCALISER_RDBMS.msg("RDBMS.Manager.POIDConnectionOpenError", e.getMessage());
        					JPOXLogger.POID.error(msg);
        					throw new JDODataStoreException(msg, e);
        				}
        				return conn;
        			}

        			public void releaseConnection()
        			{
        				try
        				{
        					if (pm.getPMFContext().getPmfConfiguration().getPoidTransactionAttribute().equalsIgnoreCase("UsePM"))
        					{
        						thisStoreMgr.releaseConnection(pm, conn);
        					}
        					else
        					{
        						boolean success = false;
        						try
        						{
        							if (!conn.getAutoCommit())
        							{
        								conn.commit();
        							}
        							success = true;
        						}
        						finally
        						{
        							if (!success)
        							{
        								if (!conn.getAutoCommit())
        								{
        									conn.rollback();
        								}
        							}
        						}
        						closeConnection(conn);
        					}
        					conn = null;
        				}
        				catch (SQLException e)
        				{
        					String msg = LOCALISER_RDBMS.msg("RDBMS.Manager.POIDConnectionCloseError", e);
        					JPOXLogger.POID.error(msg);
        					throw new JDODataStoreException(msg, e);
        				}
        			}
        		};
        		((AbstractRDBMSPoidGenerator)generator).setConnectionProvider(connProvider);
        	}
        	oid = generator.next();
        }

        AbstractPropertyMetaData fmd = cmd.getManagedFieldAbsolute(absoluteFieldNumber);
        if (fmd != null)
        {
            // replace the value of the id, but before convert the value to the field type if needed
            Object convertedValue = TypeConversionHelper.convertTo(oid, fmd.getType());
            if (convertedValue == null)
            {
                throw new JDOFatalException(LOCALISER_RDBMS.msg("RDBMS.Manager.InvalidGeneratedId", fmd.getFullFieldName(), oid));
            }
            oid = convertedValue;
        }

        if (JPOXLogger.POID.isDebugEnabled())
        {
            String fieldName;
            if (fmd != null)
            {
                fieldName = fmd.getFullFieldName();
            }
            else
            {
                fieldName = cmd.getFullClassName() + " (datastore id)";
            }
            JPOXLogger.POID.debug(LOCALISER_RDBMS.msg("RDBMS.Manager.StrategyValueGenerated", fieldName, strategy,
                generator.getClass().getName(), oid));
        }

        return oid;
    }

    /**
     * Returns a request object that will insert a row in the given table. The
     * store manager will cache the request object for re-use by subsequent
     * requests to the same table.
     * @param table The table into which to insert.
     * @param cls class of the object of the request
     * @param clr ClassLoader resolver
     * @return An insertion request object.
     */
    public InsertRequest getInsertRequest(DatastoreClass table, Class cls, ClassLoaderResolver clr)
    {
        RequestIdentifier reqID = new RequestIdentifier(table, null, RequestIdentifier.Type.INSERT, cls.getName());
        InsertRequest req = (InsertRequest) requestsByID.get(reqID);
        if (req == null)
        {
            req = new InsertRequest(table, cls, clr);
            requestsByID.put(reqID, req);
        }
        return req;
    }

    /**
     * Returns a request object that will fetch a row from the given table. The
     * store manager will cache the request object for re-use by subsequent
     * requests to the same table.
     * 
     * @param table The table from which to fetch.
     * @param fieldMetaData MetaData for the fields corresponding to the columns
     *     to be fetched.
     * @param cls class of the object of the request
     * @param clr ClassLoader resolver
     * @return A fetch request object.
     */
    public FetchRequest getFetchRequest(DatastoreClass table, AbstractPropertyMetaData[] fieldMetaData, Class cls, ClassLoaderResolver clr)
    {
        RequestIdentifier reqID = new RequestIdentifier(table, fieldMetaData, RequestIdentifier.Type.FETCH, cls.getName());
        FetchRequest req = (FetchRequest) requestsByID.get(reqID);
        if (req == null)
        {
            req = new FetchRequest(table, fieldMetaData, cls, clr);
            requestsByID.put(reqID, req);
        }
        return req;
    }

    /**
     * Returns a request object that will update a row in the given table. The
     * store manager will cache the request object for re-use by subsequent
     * requests to the same table.
     * @param table The table in which to update.
     * @param fieldMetaData The AbstractPropertyMetaData corresponding to the columns to be
     * updated. FieldMetaData whose columns exist in supertables will be
     * ignored.
     * @param cls class of the object of the request
     * @param clr ClassLoader resolver
     * @return An update request object.
     */
    public UpdateRequest getUpdateRequest(DatastoreClass table, AbstractPropertyMetaData[] fieldMetaData, Class cls, ClassLoaderResolver clr)
    {
        RequestIdentifier reqID = new RequestIdentifier(table, fieldMetaData, RequestIdentifier.Type.UPDATE, cls.getName());
        UpdateRequest req = (UpdateRequest) requestsByID.get(reqID);
        if (req == null)
        {
            req = new UpdateRequest(table, fieldMetaData, cls, clr);
            requestsByID.put(reqID, req);
        }
        return req;
    }

    /**
     * Returns a request object that will locate a row from the given table. The
     * store manager will cache the request object for re-use by subsequent
     * requests to the same table.
     * 
     * @param table The table from which to locate.
     * @param className the class name of the object of the request
     * @return A locate request object.
     */
    public LocateRequest getLocateRequest(DatastoreClass table, String className)
    {
        RequestIdentifier reqID = new RequestIdentifier(table, null, RequestIdentifier.Type.LOCATE, className);
        LocateRequest req = (LocateRequest) requestsByID.get(reqID);
        if (req == null)
        {
            req = new LocateRequest(table);
            requestsByID.put(reqID, req);
        }
        return req;
    }

    /**
     * Returns a request object that will delete a row from the given table. The
     * store manager will cache the request object for re-use by subsequent
     * requests to the same table.
     * 
     * @param table The table from which to delete.
     * @param cls class of the object of the request
     * @param clr ClassLoader resolver
     * @return A deletion request object.
     */
    public DeleteRequest getDeleteRequest(DatastoreClass table, Class cls, ClassLoaderResolver clr)
    {
        RequestIdentifier reqID = new RequestIdentifier(table, null, RequestIdentifier.Type.DELETE, cls.getName());
        DeleteRequest req = (DeleteRequest) requestsByID.get(reqID);
        if (req == null)
        {
            req = new DeleteRequest(table, cls, clr);
            requestsByID.put(reqID, req);
        }
        return req;
    }

    /**
     * Returns the column info for a column name. This should be used instead
     * of making direct calls to DatabaseMetaData.getColumns().
     * <p>
     * Where possible, this method loads and caches column info for more than
     * just the table being requested, improving performance by reducing the
     * overall number of calls made to DatabaseMetaData.getColumns() (each of
     * which usually results in one or more database queries).
     * </p>
     * @param table The table/view
     * @param conn JDBC connection to the database.
     * @param column the column
     * @return The ColumnInfo objects describing the column.
     * @see org.jpox.store.rdbms.columninfo.ColumnInfo
     * @throws SQLException
     */
    public ColumnInfo getColumnInfoForColumnName(Table table, Connection conn, DatastoreIdentifier column)
    throws SQLException
    {
        ColumnInfo colInfo = null;
        if (storeDataByClass.size() == 0)
        {
            // No SchemaData yet so go to the database directly to find any info for this table.
            colInfo = RDBMSStoreHelper.getColumnInfoForColumnName(this, table, conn, column);
            if (JPOXLogger.RDBMS_SCHEMA.isDebugEnabled())
            {
                JPOXLogger.RDBMS_SCHEMA.debug(LOCALISER_RDBMS.msg("RDBMS.Manager.ColumnInfos", table, (colInfo != null ? "" + 1 : "" + 0)));
            }
        }
        else
        {
            synchronized (this)
            {
                long now = System.currentTimeMillis();

                // Get fully-qualified name for this table (regardless of whether it is specified by user)
                DatastoreIdentifier tableId = ((AbstractTable)table).getDatastoreIdentifierFullyQualified();
                List cols = null;
                // Check if we already have valid info for this table in the cache
                if (now >= columnInfoReadTimestamp &&
                    now < columnInfoReadTimestamp + COLUMN_INFO_EXPIRATION_MS)
                {
                    cols = (List) columnInfoByColumnName.get(tableId.getIdentifier()+column.getIdentifier());
                    if (cols != null)
                    {
                        for(int i=0; i<cols.size(); i++)
                        {
                            String[] c = RDBMSStoreHelper.splitColumnIdentifierName(((RDBMSAdapter)this.getDatastoreAdapter()).getCatalogSeparator(), column.getIdentifier());
                            String columnName = column.getIdentifier();
                            if (c[RDBMSStoreHelper.TABLE_IDENTIFIER_COLUMN] != null)
                            {
                                columnName = c[RDBMSStoreHelper.TABLE_IDENTIFIER_COLUMN];
                            }                            
                            ColumnInfo ci = (ColumnInfo) cols.get(i);
                            if (ci.getColumnName().equals(columnName))
                            {
                                colInfo = ci;
                            }
                        }
                    }
                }

                // Refresh the cache if we have no/stale info for that table
                if (cols == null)
                {
                    //TODO have a cache of columnInfo by column name
                    colInfo = RDBMSStoreHelper.getColumnInfoForColumnName(this, table, conn, column);
                    if (colInfo == null)
                    {
                        JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.ColumnInfoNotFound", column.getIdentifier(), table));
                    }
                    else
                    {
                        columnInfoByColumnName.put(tableId.getIdentifier()+column.getIdentifier(), colInfo);
                        if (JPOXLogger.RDBMS_SCHEMA.isDebugEnabled())
                        {
                            JPOXLogger.RDBMS_SCHEMA.debug(LOCALISER_RDBMS.msg("RDBMS.Manager.ColumnInfo", colInfo.getColumnName(), table));
                        }
                    }
                }
            }
        }

        return colInfo;
    }
    
    /**
     * Returns the column info for a database table. This should be used instead
     * of making direct calls to DatabaseMetaData.getColumns().
     * <p>
     * Where possible, this method loads and caches column info for more than
     * just the table being requested, improving performance by reducing the
     * overall number of calls made to DatabaseMetaData.getColumns() (each of
     * which usually results in one or more database queries).
     * </p>
     * @param table The table/view
     * @param conn JDBC connection to the database.
     * @return A list of ColumnInfo objects describing the columns of the table.
     * The list is in the same order as was supplied by getColumns(). If no
     * column info is found for the given table, an empty list is returned.
     * @see org.jpox.store.rdbms.columninfo.ColumnInfo
     * @throws SQLException
     */
    public List getColumnInfoForTable(Table table, Connection conn)
    throws SQLException
    {
        List cols = null;
        if (storeDataByClass.size() == 0)
        {
            // No SchemaData yet so go to the database directly to find any info for this table.
            cols = RDBMSStoreHelper.getColumnInfoForTable(this, table, conn);
            if (JPOXLogger.RDBMS_SCHEMA.isDebugEnabled())
            {
                JPOXLogger.RDBMS_SCHEMA.debug(LOCALISER_RDBMS.msg("RDBMS.Manager.ColumnInfos", table, (cols != null ? "" + cols.size() : "" + 0)));
            }
        }
        else
        {
            synchronized (this)
            {
                long now = System.currentTimeMillis();

                // Get fully-qualified name for this table (regardless of whether it is specified by user)
                DatastoreIdentifier tableId = ((AbstractTable)table).getDatastoreIdentifierFullyQualified();

                // Check if we already have valid info for this table in the cache
                if (now >= columnInfoReadTimestamp &&
                    now < columnInfoReadTimestamp + COLUMN_INFO_EXPIRATION_MS)
                {
                    cols = (List) columnInfoByTableName.get(tableId);
                }

                // Refresh the cache if we have no/stale info for that table
                if (cols == null)
                {
                    refreshColumnInfo(table.getCatalogName(), table.getSchemaName(), conn);

                    // Finally, lookup info for the desired table in the new cache.
                    cols = (List) columnInfoByTableName.get(tableId);
                    if (cols == null)
                    {
                        cols = Collections.EMPTY_LIST;
                        JPOXLogger.RDBMS_SCHEMA.info(LOCALISER_RDBMS.msg("RDBMS.Manager.ColumnInfoNotFoundForTable", table));
                    }
                    else
                    {
                        if (JPOXLogger.RDBMS_SCHEMA.isDebugEnabled())
                        {
                            JPOXLogger.RDBMS_SCHEMA.debug(LOCALISER_RDBMS.msg("RDBMS.Manager.ColumnInfos", table, "" + cols.size()));
                        }
                    }
                }
            }
        }

        return cols;
    }

    /**
     * Returns the table info for a database schema. This should be used instead
     * of making direct calls to DatabaseMetaData.getTables().
     * <p>
     * Where possible, this method loads and caches table info for more than
     * just the table being requested, improving performance by reducing the
     * overall number of calls made to DatabaseMetaData.getTables() (each of
     * which usually results in one or more database queries).
     * </p>
     * @return A list of TableInfo objects describing the tables.
     * The list is in the same order as was supplied by getTables(). If no
     * table info, an empty list is returned.
     * @see org.jpox.store.rdbms.columninfo.TableInfo
     * @throws SQLException
     */
    public List getTableInfo()
    throws SQLException
    {
        return getTableInfo(this.schemaName);
    }

    /**
     * Returns the table info for a database schema. This should be used instead
     * of making direct calls to DatabaseMetaData.getTables(). The tables are
     * searched using the default schema name for this RDBMSManager
     * <p>
     * Where possible, this method loads and caches table info for more than
     * just the table being requested, improving performance by reducing the
     * overall number of calls made to DatabaseMetaData.getTables() (each of
     * which usually results in one or more database queries).
     * </p>
     * @param schemaNamePattern the schema name pattern
     * @return A list of TableInfo objects describing the tables.
     * The list is in the same order as was supplied by getTables(). If no
     * table info, an empty list is returned.
     * @see org.jpox.store.rdbms.columninfo.TableInfo
     * @throws SQLException
     */
    public List getTableInfo(String schemaNamePattern)
    throws SQLException
    {
        Connection conn = null;
        try
        {
            if (userName == null)
            {
                conn = FailoverUtils.getConnection(ds, false);
            }
            else
            {
                conn = FailoverUtils.getConnection(ds, userName, password, false);
            }
            return RDBMSStoreHelper.getTableInfo(this, this.catalogName, schemaNamePattern, conn);
        }
        finally
        {
            if (conn != null)
            {
                conn.close();
            }
        }
    }
    
    /**
     * Returns the column info for a database schema. This should be used instead
     * of making direct calls to DatabaseMetaData.getColumns().
     * <p>
     * Where possible, this method loads and caches table info for more than
     * just the table being requested, improving performance by reducing the
     * overall number of calls made to DatabaseMetaData.getColumns() (each of
     * which usually results in one or more database queries).
     * </p>
     * @param schemaNamePattern the schema name pattern
     * @param tableNamePattern the table name pattern
     * @return A list of ColumnInfo objects describing the columns.
     * The list is in the same order as was supplied by getColumns(). If no
     * column info, an empty list is returned.
     * @see org.jpox.store.rdbms.columninfo.ColumnInfo
     * @throws SQLException
     */
    public List getColumnInfo(String schemaNamePattern, String tableNamePattern)
    throws SQLException
    {
        Connection conn = null;
        try
        {
            if (userName == null)
            {
                conn = FailoverUtils.getConnection(ds, false);
            }
            else
            {
                conn = FailoverUtils.getConnection(ds, userName, password, false);
            }
            List cols = new ArrayList();
            ResultSet rs = ((RDBMSAdapter)dba).getColumns(conn, catalogName, schemaNamePattern, tableNamePattern);
            try
            {
                while (rs.next())
                {
                    cols.add(((RDBMSAdapter)dba).newColumnInfo(rs));
                }
            }
            finally
            {
                rs.close();
            }
            return cols;
        }
        finally
        {
            if (conn != null)
            {
                conn.close();
            }
        }
    }
        
    
    /**
     * Method to invalidate the cached column info for a table.
     * This is called when we have just added columns to the table in the schema
     * has the effect of a reload of the tables information the next time it is needed.
     * @param table The table
     */
    public void invalidateColumnInfoForTable(Table table)
    {
        DatastoreIdentifier tableId = ((AbstractTable)table).getDatastoreIdentifierFullyQualified();
        columnInfoByTableName.remove(tableId);
    }

    /**
     * Refresh or initialise the column info cache
     * <p>
     * Where possible, this method loads and caches column info for more than
     * just the table being requested, improving performance by reducing the
     * overall number of calls made to DatabaseMetaData.getColumns() (each of
     * which usually results in one or more database queries).
     * </p>
     * @param catalogName Name of catalog to refresh
     * @param schemaName Name of schema to refresh
     * @param conn JDBC connection to the database.
     * @see org.jpox.store.rdbms.columninfo.ColumnInfo
     * @throws SQLException
     */
    private void refreshColumnInfo(String catalogName, String schemaName, Connection conn)
    throws SQLException
    {
        long now = System.currentTimeMillis();

        // Extract a list of known JDO table names.
        Set table_keys = new HashSet();
        for (Iterator i = storeDataByClass.values().iterator(); i.hasNext();)
        {
            RDBMSStoreData sd = (RDBMSStoreData) i.next();
            if (sd.getDatastoreContainerObject() != null)
            {
                table_keys.add(((AbstractTable)sd.getDatastoreContainerObject()).getDatastoreIdentifierFullyQualified());
            }
        }

        // Query for column info on all tables in catalog/schema,
        // discarding info for any tables that aren't JDO tables,
        // or that are JDO tables but are already validated.
        Map cim = new HashMap();
        ResultSet rs = ((RDBMSAdapter)dba).getColumns(conn, catalogName, schemaName, null);
        try
        {
            while (rs.next())
            {
                // Construct a fully-qualified name for the table in this row of the ResultSet
                String tableIdentifierCatalogName = rs.getString(1);
                String tableIdentifierSchemaName = rs.getString(2);
                String tableIdentifierTableName = rs.getString(3);

                if (rs.wasNull() ||
                    (tableIdentifierCatalogName != null && tableIdentifierCatalogName.length() < 1))
                {
                    tableIdentifierCatalogName = null;
                }
                if (rs.wasNull() ||
                    (tableIdentifierSchemaName != null && tableIdentifierSchemaName.length() < 1))
                {
                    tableIdentifierSchemaName = null;
                }
                if (rs.wasNull() ||
                    (tableIdentifierTableName != null && tableIdentifierTableName.length() < 1))
                {
                    tableIdentifierTableName = null;
                }
                //at least the JDBC driver should return the table name
                if (tableIdentifierTableName == null)
                {
                    throw new JDODataStoreException("Invalid 'null' table name identifier returned by database. Check with your JDBC driver vendor (ref:DatabaseMetaData.getColumns).");
                }

                SQLIdentifier fullyQualifiedTableName = (SQLIdentifier)identifierFactory.newDatastoreContainerIdentifier(tableIdentifierTableName);
                fullyQualifiedTableName.setCatalogName(tableIdentifierCatalogName);
                fullyQualifiedTableName.setSchemaName(tableIdentifierSchemaName);

                if (table_keys.contains(fullyQualifiedTableName))
                {
                    // Process this row if it is for one of our tables
                    List l = (List) cim.get(fullyQualifiedTableName);
                    if (l == null)
                    {
                        // Add an entry for the table if none exists
                        l = new ArrayList();
                        cim.put(fullyQualifiedTableName, l);
                    }

                    // Add the column for this tables entry
                    l.add(((RDBMSAdapter)dba).newColumnInfo(rs));
                }
            }
        }
        finally
        {
            rs.close();
        }

        if (JPOXLogger.RDBMS_SCHEMA.isDebugEnabled())
        {
            JPOXLogger.RDBMS_SCHEMA.debug(LOCALISER_RDBMS.msg("RDBMS.Manager.ColumnInfoLoaded", catalogName, schemaName,
                "" + cim.size(), "" + (System.currentTimeMillis() - now)));
        }

        // Replace the old cache (if any) with the new one
        if (columnInfoByTableName != null && columnInfoByTableName != cim)
        {
            // help the gargabe collector
            columnInfoByTableName.clear();
        }
        columnInfoByTableName = cim;
        columnInfoReadTimestamp = now;        
    }

    /**
     * Resolves an identifier macro. The public fields <var>className</var>,
     * <var>fieldName </var>, and <var>subfieldName </var> of the given macro
     * are taken as inputs, and the public <var>value </var> field is set to the
     * SQL identifier of the corresponding database table or column.
     * @param im The macro to resolve.
     * @param clr The ClassLoaderResolver
     */
    public void resolveIdentifierMacro(MacroString.IdentifierMacro im, ClassLoaderResolver clr)
    {
        DatastoreClass ct = getDatastoreClass(im.className, clr);
        if (im.fieldName == null)
        {
            im.value = ct.getIdentifier().toString();
            return;
        }

        JavaTypeMapping m;
        if (im.fieldName.equals("this"))
        {
            if (!(ct instanceof ClassTable))
            {
                throw new JDOUserException(LOCALISER_RDBMS.msg("RDBMS.Manager.TableIDColumnError", im.className));
            }

            if (im.subfieldName != null)
            {
                throw new JDOUserException(LOCALISER_RDBMS.msg("RDBMS.Manager.FieldTableError", im.className, im.fieldName, im.subfieldName));
            }
            m = ((DatastoreContainerObject) ct).getIDMapping();
        }
        else
        {
            AbstractPropertyMetaData fmd = getMetaDataManager().getMetaDataForField(im.className, im.fieldName, clr);
            m = ct.getFieldMapping(fmd);
            DatastoreContainerObject t = getDatastoreContainerObject(fmd);
            if (im.subfieldName == null)
            {
                if (t != null)
                {
                    im.value = t.getIdentifier().toString();
                    return;
                }
            }
            else
            {
                if (t instanceof SetTable)
                {
                    SetTable st = (SetTable) t;
                    if (im.subfieldName.equals("owner"))
                    {
                        m = st.getOwnerMapping();
                    }
                    else if (im.subfieldName.equals("element"))
                    {
                        m = st.getElementMapping();
                    }
                    else
                    {
                        throw new JDOUserException(LOCALISER_RDBMS.msg(
                                "RDBMS.Manager.InvalidPseudoFieldNameForSet",
                                im.subfieldName, im));
                    }
                }
                else if (t instanceof ListTable)
                {
                    ListTable lt = (ListTable) t;
                    if (im.subfieldName.equals("owner"))
                    {
                        m = lt.getOwnerMapping();
                    }
                    else if (im.subfieldName.equals("element"))
                    {
                        m = lt.getElementMapping();
                    }
                    else if (im.subfieldName.equals("index"))
                    {
                        m = lt.getOrderMapping();
                    }
                    else
                    {
                        throw new JDOUserException(LOCALISER_RDBMS.msg(
                                "RDBMS.Manager.InvalidPseudoFieldNameForSet",
                                im.subfieldName, im));
                    }
                }
                else if (t instanceof MapTable)
                {
                    MapTable mt = (MapTable) t;
                    if (im.subfieldName.equals("owner"))
                    {
                        m = mt.getOwnerMapping();
                    }
                    else if (im.subfieldName.equals("key"))
                    {
                        m = mt.getKeyMapping();
                    }
                    else if (im.subfieldName.equals("value"))
                    {
                        m = mt.getValueMapping();
                    }
                    else
                    {
                        throw new JDOUserException(LOCALISER_RDBMS.msg(
                                "RDBMS.Manager.InvalidPseudoFieldNameForMap",
                                im.subfieldName, im));
                    }
                }
                else
                {
                    throw new JDOUserException(LOCALISER_RDBMS.msg(
                            "RDBMS.Manager.FieldTableError", im.className,
                            im.fieldName, im.subfieldName));
                }
            }
        }
        im.value = ((Column)m.getDataStoreMapping(0).getDatastoreField()).getIdentifier().toString();
    }

    /**
     * Accessor for the SQL identifier for a table defined by the specified class.
     * Consults the StoreData for the class. If the class isn't known it generates the table name.
     * @param cmd ClassMetaData for the class
     * @return The SQL identifier for the table
     */
    private DatastoreIdentifier getTableIdentifier(ClassMetaData cmd, ClassLoaderResolver clr)
    {
        if (storeDataByClass.containsKey(cmd.getFullClassName()))
        {
            RDBMSStoreData sd = (RDBMSStoreData) storeDataByClass.get(cmd.getFullClassName());
            if (sd.getDatastoreIdentifier() != null)
            {
                return sd.getDatastoreIdentifier();
            }
        }
        return identifierFactory.newDatastoreContainerIdentifier(clr, cmd);
    }

    /**
     * Accessor for the SQL identifier for a (join) table defined by the specified field.
     * Consults the StoreData for the field. If the field isn't known it generates the table name.
     * @param fmd AbstractPropertyMetaData for the field with the SCO needing a table
     * @return The SQL identifier for the table
     */
    private DatastoreIdentifier getTableIdentifier(AbstractPropertyMetaData fmd, ClassLoaderResolver clr)
    {
        if (storeDataByClass.containsKey(fmd))
        {
            RDBMSStoreData sd = (RDBMSStoreData) storeDataByClass.get(fmd);
            if (sd.getDatastoreIdentifier() != null)
            {
                return sd.getDatastoreIdentifier();
            }
        }
        return identifierFactory.newDatastoreContainerIdentifier(clr, fmd);
    }

    // ============================ Inner classes ==============================

    /**
     * An abstract base class for RDBMSManager transactions that perform some
     * management function on the database.
     * <p>
     * Management transactions may be retried in the face of SQL exceptions to
     * work around failures caused by transient conditions, such as DB
     * deadlocks.
     * </p>
     */
    private abstract class MgmtTransaction
    {
        protected final int isolationLevel;
        protected final int maxRetries;
        protected Connection conn;

        /**
         * Constructs a new management transaction having the given isolation
         * level.
         * @param isolationLevel One of the isolation level constants from
         * java.sql.Connection.
         */
        private MgmtTransaction(int isolationLevel)
        {
            this.isolationLevel = isolationLevel;
            maxRetries = pmfContext.getPmfConfiguration().getDatastoreClassAdditionMaxRetries();
        }

        /**
         * Returns a description of the management transaction. Subclasses
         * should override this method so that transaction failures are given an
         * appropriate exception message.
         * 
         * @return A description of the management transaction.
         */
        public abstract String toString();

        /**
         * Implements the body of the transaction.
         * @param clr the ClassLoaderResolver
         * @exception SQLException Thrown if the transaction fails due to a
         * database error that should allow the entire transaction to be
         * retried.
         */
        protected abstract void run(ClassLoaderResolver clr)
        throws SQLException;
        
        /**
         * Obtains the current connection for the transaction. Creates
         * one if needed
         * @return the connection
         */
        protected Connection getCurrentConnection() throws SQLException 
        {
            if (conn == null)
            {
                conn = getConnection(isolationLevel);
            }
            return conn;
        }
        
        /**
         * Executes the transaction.
         * <p>
         * A database connection is acquired and the
         * {@link #execute(ClassLoaderResolver)}method is invoked.
         * If the selected isolation level is not Connection.TRANSACTION_NONE,
         * then commit() or rollback() is called on the connection according to
         * whether the invocation succeeded or not. If the invocation failed the
         * sequence is repeated, up to a maximum of <var>maxRetries </var>
         * times, configurable by the system property org.jpox.store.maxRetries.
         * @param clr the ClassLoaderResolver
         * @exception JDODataStoreException If a SQL exception occurred even
         * after <var>maxRetries </var> attempts.
         */
        public final void execute(ClassLoaderResolver clr)
        {
            int attempts = 0;
            for (;;)
            {
                try
                {
                    try
                    {
                        boolean succeeded = false;
                        try
                        {
                            run(clr);
                            succeeded = true;
                        }
                        finally
                        {
                            if (conn != null)
                            {
                                if (isolationLevel != Connection.TRANSACTION_NONE)
                                {
                                    if (!conn.getAutoCommit())
                                    {
                                        if (succeeded)
                                        {
                                            conn.commit();
                                        }
                                        else
                                        {
                                            conn.rollback();
                                        }
                                    }
                                }
                            }
                        }
                    }
                    finally
                    {
                        if (conn != null)
                        {
                            closeConnection(conn);
                            conn = null;
                        }
                    }
                    break;
                }
                catch (SQLException e)
                {
                    if (++attempts >= maxRetries)
                    {
                        throw new JDODataStoreException("SQL exception: " + this, e);
                    }
                }
            }
        }
    }

    /**
     * A management transaction that adds a set of classes to the RDBMSManager,
     * making them usable for persistence.
     * <p>
     * This class embodies the work necessary to activate a persistent class and
     * ready it for storage management. It is the primary mutator of a
     * RDBMSManager.
     * </p>
     * <p>
     * Adding classes is an involved process that includes the creation and/or
     * validation in the database of tables, views, and table constraints, and
     * their corresponding Java objects maintained by the RDBMSManager. Since
     * it's a management transaction, the entire process is subject to retry on
     * SQL exceptions. It is responsible for ensuring that the procedure either
     * adds <i>all </i> of the requested classes successfully, or adds none of
     * them and preserves the previous state of the RDBMSManager exactly as it
     * was.
     * </p>
     */
    private class ClassAdder extends MgmtTransaction
    {
        /** join table for Set **/
        public static final int JOIN_TABLE_SET = 1;
        /** join table for List **/
        public static final int JOIN_TABLE_LIST = 2;
        /** join table for Map **/
        public static final int JOIN_TABLE_MAP = 3;
        /** join table for Array **/
        public static final int JOIN_TABLE_ARRAY = 4;

        /** Optional writer to dump the DDL for any classes being added. */
        private FileWriter ddlWriter = null;

        /** tracks the SchemaData currently being added - used to roll-back the AutoStart added classes **/
        private HashSet schemaDataAdded = new HashSet();
        private final String[] classNames;

        /**
         * Constructs a new class adder transaction that will add the given classes to the RDBMSManager.
         * @param classNames Names of the (initial) class(es) to be added.
         * @param writer Optional writer for DDL when we want the DDL outputting to file instead of creating the tables
         */
        private ClassAdder(String[] classNames, FileWriter writer)
        {
            super(((RDBMSAdapter)dba).getTransactionIsolationForSchemaCreation());
            this.ddlWriter = writer;
            this.classNames = filterSimpleClasses(classNames);
        }

        private String[] filterSimpleClasses(String[] classNames)
        {
            int filteredClasses = 0;
            for (int i = 0; i < classNames.length; ++i)
            {
                if (getPMFContext().getTypeManager().isSupportedType(classNames[i]))
                {
                    classNames[i] = null;
                    ++filteredClasses;
                }
            }
            if (filteredClasses == 0)
            {
                return classNames;
            }
            String[] restClasses = new String[classNames.length - filteredClasses];
            int m = 0;
            for (int i = 0; i < classNames.length; ++i)
            {
                if (classNames[i] != null)
                {
                    restClasses[m++] = classNames[i];
                }
            }
            return restClasses;
        }

        /**
         * Method to give a string version of this object.
         * @return The String version of this object.
         */
        public String toString()
        {
            return LOCALISER_RDBMS.msg("RDBMS.Manager.AddClassesToSchema", catalogName, schemaName);
        }

        /**
         * Method to perform the action using the specified connection to the datastore.
         * @param clr the ClassLoaderResolver
         * @throws SQLException Thrown if an error occurs in execution.
         */
        protected void run(ClassLoaderResolver clr)
        throws SQLException
        {
            if (classNames.length == 0)
            {
                return;
            }

            synchronized (CLASSADDER_MUTEX)
            {
                classAdder = this;
                try
                {
                    addClassTablesAndValidate(classNames, clr);
                }
                finally
                {
                    classAdder = null;
                }
            }
        }

        /**
         * Called by RDBMSManager.addClasses() when it has been recursively
         * re-entered. This just adds table objects for the requested additional 
         * classes and returns.
         * @param classNames Names of the (additional) class(es) to be added.
         * @param clr the ClassLoaderResolver
         */
        private void addClasses(String[] classNames, ClassLoaderResolver clr)
        {
            // Filter out any supported classes
            classNames = filterSimpleClasses(classNames);
            if (classNames.length == 0)
            {
                return;
            }

            try
            {
                if (getCurrentConnection() == null)
                {
                    throw new IllegalStateException(LOCALISER_RDBMS.msg("RDBMS.Manager.AddClassesError"));
                }
            }
            catch(SQLException e)
            {
                throw new JDODataStoreException("SQL exception: " + this, e);
            }

            // Add the tables for these additional classes
            addClassTables(classNames, clr);
        }
        
        private int addClassTablesRecursionCounter = 0;

        /**
         * Adds a new table object (ie ClassTable or ClassView).
         * <p>
         * This doesn't initialize or validate the tables, it just adds the
         * table objects to the RDBMSManager's internal data structures.
         * </p>
         *
         * @param cmd the ClassMetaData
         * @param clr the ClassLoaderResolver
         */
        private void addClassTable(ClassMetaData cmd, ClassLoaderResolver clr)
        {
            // Only add tables for "PERSISTENCE_CAPABLE" classes
            if (cmd.getPersistenceModifier() != ClassPersistenceModifier.PERSISTENCE_CAPABLE)
            {
                return;
            }
            if (cmd.getIdentityType() == IdentityType.NONDURABLE)
            {
                if (cmd.hasExtension("requires-table") && cmd.getValueForExtension("requires-table") != null && 
                        cmd.getValueForExtension("requires-table").equalsIgnoreCase("false"))
                {
                    return;
                }
            }
            RDBMSStoreData sd = (RDBMSStoreData) storeDataByClass.get(cmd.getFullClassName());
            if (sd == null)
            {
                // For application-identity classes with
                // user-defined identities we
                // check for use of the objectid-class in different
                // inheritance trees
                // We prevent this to avoid problems later on.
                // The JDO2 builtin objectid-classes are allowed to
                // be duplicated.
                if (cmd.getIdentityType() == IdentityType.APPLICATION)
                {
                    if (!cmd.usesSingleFieldIdentityClass())
                    {
                        // Check whether this class has the same
                        // base persistable class as the others
                        // using the PK. If not, then throw an error
                        String baseClassWithMetaData = cmd.getBaseAbstractClassMetaData().getFullClassName();
                        Collection c = (Collection) storeDataByAppIdClass.get(cmd.getObjectidClass());
                        if (c != null && c.size() > 0)
                        {
                            // We already have at least 1 class
                            // using the same app id PK class
                            // so check if it is has the same
                            // persistable root class.
                            boolean in_same_tree = false;
                            String sample_class_in_other_tree = null;

                            Iterator id_iter = c.iterator();
                            while (id_iter.hasNext())
                            {
                                RDBMSStoreData id_sd = (RDBMSStoreData) id_iter.next();
                                String otherClassBaseClass = 
                                    ((AbstractClassMetaData) id_sd.getMetaData()).getBaseAbstractClassMetaData().getFullClassName();
                                if (otherClassBaseClass.equals(baseClassWithMetaData))
                                {
                                    in_same_tree = true;
                                    break;
                                }
                                sample_class_in_other_tree = id_sd.getName();
                            }

                            if (!in_same_tree)
                            {
                                String error_msg = LOCALISER_RDBMS.msg("RDBMS.Manager.ObjectIdClassDuplicated", cmd.getFullClassName(), 
                                    cmd.getObjectidClass(), sample_class_in_other_tree);
                                JPOXLogger.GENERAL.error(error_msg);
                                throw new JDOUserException(error_msg);
                            }
                        }
                    }
                }

                if (cmd.isEmbeddedOnly())
                {
                    JPOXLogger.RDBMS.info(LOCALISER.msg("StoreManager.NoTableForEmbeddedClass", cmd.getFullClassName()));
                    // Nothing to do. Only persisted as SCO.
                }
                else
                {
                    InheritanceMetaData imd = cmd.getInheritanceMetaData();
                    RDBMSStoreData sdNew = null;
                    if (imd.getStrategyValue() == InheritanceStrategy.SUBCLASS_TABLE)
                    {
                        // Table mapped into the table(s) of subclass(es)
                        // Just add the SchemaData entry with no table - managed by subclass
                        sdNew = new RDBMSStoreData(cmd, null, false);
                        registerStoreData(sdNew);
                    }
                    else if (imd.getStrategyValue() == InheritanceStrategy.NEW_TABLE)
                    {
                        // Table managed by this class
                        // Generate an identifier for the table required
                        DatastoreIdentifier tableName = getTableIdentifier(cmd, clr);

                        // Check that the required table isn't already in use
                        StoreData[] existingStoreData = getStoreDataForDatastoreContainerObject(tableName);
                        if (existingStoreData != null)
                        {
                            String existingClass = null;
                            for (int j=0;j<existingStoreData.length;j++)
                            {
                                if (!existingStoreData[j].getName().equals(cmd.getFullClassName()))
                                {
                                    existingClass = existingStoreData[j].getName();
                                    break;
                                }
                            }
                            // Give a warning and then create a new instance of the table (mapped to the same datastore object)
                            if (existingClass != null)
                            {
                                String msg = LOCALISER_RDBMS.msg("RDBMS.Manager.TableAlreadyInUse", cmd.getFullClassName(), 
                                    tableName.getIdentifier(), existingClass);
                                JPOXLogger.RDBMS.warn(msg);
                            }
                        }

                        // Create the table to use for this class
                        DatastoreClass t = null;
                        boolean hasViewDef = false;
                        if (dba.getVendorID() != null)
                        {
                            hasViewDef = cmd.hasExtension("view-definition" + '-' + dba.getVendorID());
                        }
                        if (!hasViewDef)
                        {
                            hasViewDef = cmd.hasExtension("view-definition");
                        }
                        if (hasViewDef)
                        {
                            t = new ClassView(tableName, RDBMSManager.this, cmd);
                        }
                        else
                        {
                            t = new ClassTable(tableName, RDBMSManager.this, cmd);
                        }

                        sdNew = new RDBMSStoreData(cmd, t, true);
                        registerStoreData(sdNew);

                        // must be initialized after registering, to avoid StackOverflowError
                        ((Table) t).preInitialize(clr);
                    }
                    else if (imd.getStrategyValue() == InheritanceStrategy.SUPERCLASS_TABLE)
                    {
                        // Table mapped into table of superclass
                        // Find the superclass - should have been created first
                        AbstractClassMetaData[] managingCmds = getClassesManagingTableForClass(cmd, clr);
                        DatastoreContainerObject superTable = null;
                        if (managingCmds != null && managingCmds.length == 1)
                        {
                            RDBMSStoreData superData = (RDBMSStoreData) storeDataByClass.get(managingCmds[0].getFullClassName());
                            if (superData != null)
                            {
                                // Specify the table if it already
                                // exists
                                superTable = superData.getDatastoreContainerObject();
                            }
                            sdNew = new RDBMSStoreData(cmd, superTable, false);
                            registerStoreData(sdNew);
                        }
                        else
                        {
                            String msg = LOCALISER_RDBMS.msg("RDBMS.Manager.UnableToFindSuperclassTable", cmd.getFullClassName());
                            JPOXLogger.JDO.error(msg);
                            throw new JDOUserException(msg);
                        }
                    }
                    schemaDataAdded.add(sdNew);
                }
            }
        }
        
        /**
         * Adds a new table object (ie ClassTable or ClassView) for every
         * class in the given list. These classes
         * <ol>
         * <li>require a table</li>
         * <li>do not yet have a table initialized in the store manager.</li>
         * </ol>
         * <p>
         * This doesn't initialize or validate the tables, it just adds the
         * table objects to the RDBMSManager's internal data structures.
         * </p>
         *
         * @param classNames Names of class(es) whose tables are to be added.
         * @param clr the ClassLoaderResolver
         */
        private void addClassTables(String[] classNames, ClassLoaderResolver clr)
        {
            addClassTablesRecursionCounter += 1;
            try
            {
                Iterator iter = getReferencedClasses(classNames, clr).iterator();
                try
                {
                    if (starter != null && starterInitialised && !starter.isOpen())
                    {
                        starter.open();
                    }

                    // Pass through the classes and create necessary tables
                    while (iter.hasNext())
                    {
                        ClassMetaData cmd = (ClassMetaData) iter.next();
                        addClassTable(cmd, clr);
                    }

                    // For data where the table wasn't defined, make a second
                    // pass
                    // This is necessary where a subclass uses
                    // "superclass-table" and the superclass' table
                    // hadn't been defined at the point of adding this class
                    Iterator addedIter = schemaDataAdded.iterator();
                    while (addedIter.hasNext())
                    {
                        RDBMSStoreData data = (RDBMSStoreData) addedIter.next();
                        if (data.getDatastoreContainerObject() == null && data.isFCO())
                        {
                            AbstractClassMetaData cmd = (AbstractClassMetaData) data.getMetaData();
                            InheritanceMetaData imd = cmd.getInheritanceMetaData();
                            if (imd.getStrategyValue() == InheritanceStrategy.SUPERCLASS_TABLE)
                            {
                                AbstractClassMetaData[] managingCmds = getClassesManagingTableForClass(cmd, clr);
                                DatastoreClass superTable = null;
                                if (managingCmds != null && managingCmds.length == 1)
                                {
                                    RDBMSStoreData superData = (RDBMSStoreData) storeDataByClass.get(managingCmds[0].getFullClassName());
                                    if (superData == null)
                                    {
                                        String msg = LOCALISER_RDBMS.msg("RDBMS.Manager.UnableToFindSuperclassTable", cmd.getFullClassName());
                                        JPOXLogger.JDO.error(msg);
                                        throw new JDOUserException(msg);
                                    }
                                    superTable = (DatastoreClass) superData.getDatastoreContainerObject();
                                    data.setDatastoreContainerObject(superTable);
                                }
                            }
                        }
                    }
                }
                finally
                {
                    if (starter != null && starterInitialised && starter.isOpen() && addClassTablesRecursionCounter <= 1)
                    {
                        starter.close();
                    }
                }

            }
            finally
            {
                addClassTablesRecursionCounter -= 1;
            }
        }

        /**
         * Returns a List of {@link ClassMetaData}objects representing the set
         * of classes consisting of the requested classes plus any and all other
         * classes they may reference, directly or indirectly. The returned list
         * is ordered by dependency.
         * @param classNames Array of names of persistence-capable classes.
         * @param clr the ClassLoaderResolver
         * @return A List of <tt>ClassMetaData</tt> objects.
         * @exception org.jpox.exceptions.ClassNotPersistenceCapableException If
         * any of the given classes is not persistence-capable.
         * @exception MetaDataForPersistenceCapableClassNotReachableException If
         * any of the given metadata for the classes can't be reached in the
         * classpath.
         */
        private List getReferencedClasses(String[] classNames,
                ClassLoaderResolver clr)
        {
            List cmds = new ArrayList();
            for (int i = 0; i < classNames.length; ++i)
            {
                Class clazz = clr.classForName(classNames[i]);
                if (clazz == null || !clazz.isInterface())
                {
                    AbstractClassMetaData cmd = getMetaDataManager().getMetaDataForClass(classNames[i], clr);
                    if (cmd == null)
                    {
                        JPOXLogger.RDBMS_SCHEMA.warn(LOCALISER_RDBMS.msg("RDBMS.Manager.AddClassesInvalidClassError", classNames[i]));
                        throw new MetaDataForPersistenceCapableClassNotReachableException(classNames[i]);
                    }
                    cmds.addAll(getMetaDataManager().getReferencedClassMetaData(cmd, dba.getVendorID(), clr));
                }
            }
            return cmds;
        }

        /**
         * Remove duplicated tables from the list
         * @param newTables the list of DatastoreContainerObject
         * @return a distinct list with DatastoreContainerObject  
         */
        private List removeDuplicateTablesFromList(List newTables)
        {
            List result = new ArrayList();
            Set uniqueTables = new TreeSet(new Comparator()
            {
                /*
                 * (non-Javadsoc)
                 * @see java.util.Comparator#compare(java.lang.Object,
                 * java.lang.Object)
                 */
                public int compare(Object o1, Object o2)
                {
                    DatastoreContainerObject t1 = (DatastoreContainerObject) o1;
                    DatastoreContainerObject t2 = (DatastoreContainerObject) o2;
                    return t1.getIdentifier().getIdentifier().compareTo(t2.getIdentifier().getIdentifier());
                }
            });
            uniqueTables.addAll(newTables);
            result.addAll(uniqueTables);
            return result;
        }

        /**
         * Initialisation of tables. Updates the internal representation of the
         * table to match what is required for the class(es).
         * Each time a table object is initialized, it may cause other
         * associated table objects to be added (via callbacks to addClasses())
         * so the loop is repeated until no more initialisation is needed.
         * @param classNames String array of class names
         * @param clr the ClassLoaderResolver
         * @return an array of List where index == 0 has a list of the tables created
         *                                index == 1 has a list of the views created
         */
        private List[] initializeClassTables(String[] classNames, ClassLoaderResolver clr)
        {
            List tablesToValidate = new ArrayList();
            List viewsToValidate = new ArrayList();
            boolean someNeededInitialization;
            List recentlyInitilized = new ArrayList(); 
            do
            {
                someNeededInitialization = false;
                RDBMSStoreData[] rdbmsStoreData = (RDBMSStoreData[]) storeDataByClass.values().toArray(new RDBMSStoreData[storeDataByClass.size()]);
                for (int i=0; i<rdbmsStoreData.length; i++)
                {
                    if (rdbmsStoreData[i].hasTable())
                    {
                        Table t = (Table)rdbmsStoreData[i].getDatastoreContainerObject();

                        // Any classes managed by their own table needing initialising
                        if (!t.isInitialized())
                        {
                            t.initialize(clr);
                            recentlyInitilized.add(t);
                            if (t instanceof ViewImpl)
                            {
                                viewsToValidate.add(t);
                            }
                            else
                            {
                                tablesToValidate.add(t);
                            }
                            someNeededInitialization = true;
                        }

                        // Any classes that are managed by other tables needing initialising
                        if (!rdbmsStoreData[i].isTableOwner() && !((ClassTable)t).managesClass(rdbmsStoreData[i].getName()))
                        {
                            ((ClassTable)t).manageClass((ClassMetaData)rdbmsStoreData[i].getMetaData(), clr);
                            if (!tablesToValidate.contains(t))
                            {
                                tablesToValidate.add(t);
                            }
                            someNeededInitialization = true;
                        }
                    }
                    else
                    {
                        // Nothing to do for cases without their own table ("subclass-table" strategy) since
                        // the table is initialised to contain those fields by the subclass.
                    }
                }
            }
            while (someNeededInitialization);
            for( int j=0; j<recentlyInitilized.size(); j++)
            {
                ((Table)recentlyInitilized.get(j)).postInitialize(clr);
            }
            return new List[] { tablesToValidate, viewsToValidate };
        }

        /**
         * Validate tables
         * @param tablesToValidate list of TableImpl to validate
         * @param clr the ClassLoaderResolver
         * @return an array of List where index == 0 has a list of the tables created
         *                                index == 1 has a list of the contraints created
         *                                index == 2 has a list of the auto creation errors 
         * @throws SQLException
         */
        private List[] performTablesValidation(List tablesToValidate, ClassLoaderResolver clr) throws SQLException
        {
            List autoCreateErrors = new ArrayList();
            List tableConstraintsCreated = new ArrayList();
            List tablesCreated = new ArrayList();

            if (ddlWriter != null)
            {
                // we have duplicate JDOTable objects in newBaseTables for
                // bidirectional associations, each create
                // when traversing from one side of the association. When
                // autocreating tables,
                // the second one will validate itself against the DB and
                // normally find that it
                // already exists. When the DDL is dumped into a file, the
                // second table won't find itself in the DB
                // and then generate the same DDL again, which we don't
                // want.
                // So remove all duplicate JDOTables before autocreating.
                tablesToValidate = removeDuplicateTablesFromList(tablesToValidate);
            }

            // Table existence and validation.
            // a). Check for existence of the table
            // b). If autocreate, create the table if necessary
            // c). If validate, validate the table

            Iterator i = tablesToValidate.iterator();
            while (i.hasNext())
            {
                TableImpl t = (TableImpl) i.next();

                boolean columnsValidated = false;
                if (checkExistTablesOrViews)
                {
                    if (ddlWriter != null)
                    {
                        try
                        {
                            if (t instanceof ClassTable)
                            {
                                ddlWriter.write("-- Table " + t.toString() + 
                                    " for classes " + StringUtils.objectArrayToString(((ClassTable)t).getManagedClasses()) + "\n");
                            }
                            else if (t instanceof JoinTable)
                            {
                                ddlWriter.write("-- Table " + t.toString() + " for join relationship\n");
                            }
                        }
                        catch (IOException ioe)
                        {
                            JPOXLogger.RDBMS_DDL.error("error writing DDL into file", ioe);
                        }
                    }
                    if (t.exists(getCurrentConnection(), autoCreateTables))
                    {
                        tablesCreated.add(t);
                        columnsValidated = true;
                    }
                    else
                    {
                        // Table wasn't just created, so do any autocreate of columns necessary
                        if (t.isInitializedModified() || autoCreateColumns)
                        {
                            // Check for existence of the required columns and add where required
                            t.validateColumns(getCurrentConnection(), false, autoCreateColumns, autoCreateErrors);
                            columnsValidated = true;
                        }
                    }
                }
                if (validateTables)
                {
                    // Check down to the column structure where required
                    t.validate(getCurrentConnection(), validateColumns, false, autoCreateErrors);
                }
                else if (!columnsValidated && pmfContext.getPmfConfiguration().getInitializePrimaryKeyColumnInfo())
                {
                    t.initializeColumnInfoForPrimaryKeyColumns(getCurrentConnection());
                }
                else if (!columnsValidated)
                {
                    t.initializeColumnInfoFromDatastore(getCurrentConnection());
                }

                // Discard any cached column info used to validate the table
                columnInfoByTableName.remove(t.getDatastoreIdentifierFullyQualified());
            }

            // Table constraint existence and validation
            // a). Check for existence of the constraint
            // b). If autocreate, create the constraint if necessary
            // c). If validate, validate the constraint
            // Constraint processing is done as a separate step from table processing
            // since the constraints are dependent on tables being available
            i = tablesToValidate.iterator();
            while (i.hasNext())
            {
                TableImpl t = (TableImpl) i.next();
                if (validateConstraints || autoCreateConstraints)
                {
                    if (ddlWriter != null)
                    {
                        try
                        {
                            ddlWriter.write("-- Constraints for table " + t.toString() + "\n");
                        }
                        catch (IOException ioe)
                        {
                            JPOXLogger.RDBMS_DDL.error("error writing DDL into file", ioe);
                        }
                    }
                    // TODO : split this method into checkExistsConstraints and validateConstraints
                    if (t.validateConstraints(getCurrentConnection(), autoCreateConstraints, autoCreateErrors, clr))
                    {
                        tableConstraintsCreated.add(t);
                    }
                    if (ddlWriter != null)
                    {
                        try
                        {
                            ddlWriter.write("\n");
                        }
                        catch (IOException ioe)
                        {
                            JPOXLogger.RDBMS_DDL.error("error writing DDL into file", ioe);
                        }
                    }
                }
            }
            return new List[] { tablesCreated, tableConstraintsCreated, autoCreateErrors };
        }

        /**
         * Validate views
         * @param viewsToValidate list of ViewImpl to validate
         * @return an array of List where index == 0 has a list of the views created
         *                                index == 1 has a list of the auto creation errors 
         * @throws SQLException
         */
        private List[] performViewsValidation(List viewsToValidate) throws SQLException
        {
            List viewsCreated = new ArrayList();
            List autoCreateErrors = new ArrayList();
            // View existence and validation.
            // a). Check for existence of the view
            // b). If autocreate, create the view if necessary
            // c). If validate, validate the view
            Iterator i = viewsToValidate.iterator();
            while (i.hasNext())
            {
                ViewImpl v = (ViewImpl) i.next();
                if (checkExistTablesOrViews)
                {
                    if (v.exists(getCurrentConnection(), autoCreateTables))
                    {
                        viewsCreated.add(v);
                    }
                }
                if (validateTables)
                {
                    v.validate(getCurrentConnection(), true, false, autoCreateErrors);
                }

                // Discard any cached column info used to validate the view
                columnInfoByTableName.remove(v.getDatastoreIdentifierFullyQualified());
            }
            return new List[] { viewsCreated, autoCreateErrors };
        }

        /**
         * Rollback / Compensate schema creation by dropping tables, views, constraints and
         * deleting entries in the auto start mechanism
         * @param schemaDataByClassPrev the memory image of schema data beforing running it
         * @param schemaDataByClassAppIDPrev the memory image of schema data beforing running it
         * @param viewsCreated the views created that must be dropped
         * @param tableConstraintsCreated the constraints created that must be dropped
         * @param tablesCreated the tables created that must be dropped
         */
        private void rollbackSchemaCreation(Map schemaDataByClassPrev, MultiMap schemaDataByClassAppIDPrev,
                                            List viewsCreated, List tableConstraintsCreated, List tablesCreated)
        {
            if (JPOXLogger.RDBMS_SCHEMA.isDebugEnabled())
            {
                JPOXLogger.RDBMS_SCHEMA.debug(LOCALISER_RDBMS.msg("RDBMS.Manager.AddTablesRollback"));
            }
            storeDataByClass = schemaDataByClassPrev;
            storeDataByAppIdClass = schemaDataByClassAppIDPrev;

            // Tables, table constraints, and views get removed in the
            // reverse order from which they were created.
            try
            {
                if( viewsCreated != null)
                {
                    ListIterator li = viewsCreated.listIterator(viewsCreated.size());
                    while (li.hasPrevious())
                    {
                        ((ViewImpl) li.previous()).drop(getCurrentConnection());
                    }
                }
                if( tableConstraintsCreated != null)
                {
                    ListIterator li = tableConstraintsCreated.listIterator(tableConstraintsCreated.size());
                    while (li.hasPrevious())
                    {
                        ((TableImpl) li.previous()).dropConstraints(getCurrentConnection());
                    }
                }
                if( tablesCreated != null)
                {
                    ListIterator li = tablesCreated.listIterator(tablesCreated.size());
                    while (li.hasPrevious())
                    {
                        ((TableImpl) li.previous()).drop(getCurrentConnection());
                    }
                }
            }
            catch (Exception e)
            {
                JPOXLogger.RDBMS_SCHEMA.warn(LOCALISER_RDBMS.msg(
                    "RDBMS.Manager.AddTablesRollbackError", e));
            }

            // AutoStarter - Remove all classes from the supported list
            // that were added in this pass.
            if (starter != null && starterInitialised)
            {
                try
                {
                    if (!starter.isOpen())
                    {
                        starter.open();
                    }
                    Iterator schema_added_iter = schemaDataAdded.iterator();
                    while (schema_added_iter.hasNext())
                    {
                        RDBMSStoreData sd=(RDBMSStoreData)schema_added_iter.next();
                        starter.deleteClass(sd.getName());
                    }                            
                }
                finally
                {
                    if (starter.isOpen())
                    {
                        starter.close();
                    }
                }
            }
        }

        /**
         * Verify the list of errors, log the errors and raise JDODataStoreException when fail on error is enabled
         * @param autoCreateErrors the list of Throwable 
         */
        private void verifyErrors(List autoCreateErrors)
        {
            if (autoCreateErrors.size() > 0)
            {
                // Print out all errors found during auto-creation/validation
                Iterator errorsIter = autoCreateErrors.iterator();
                while (errorsIter.hasNext())
                {
                    Throwable exc = (Throwable)errorsIter.next();
                    if (autoCreateWarnOnError)
                    {
                        JPOXLogger.RDBMS.warn(LOCALISER_RDBMS.msg("RDBMS.Manager.AddClassesExceptionError", exc));
                    }
                    else
                    {
                        JPOXLogger.RDBMS.error(LOCALISER_RDBMS.msg("RDBMS.Manager.AddClassesExceptionError", exc));
                    }
                }
                if (!autoCreateWarnOnError)
                {
                    throw new JDODataStoreException(LOCALISER_RDBMS.msg("RDBMS.Manager.AutoCreateError"), 
                        (Throwable[])autoCreateErrors.toArray(new Throwable[autoCreateErrors.size()]));
                }
            }
        }
        
        /**
         * Adds a new table object (ie ClassTable or ClassView) for every class
         * in the given list that 1) requires an extent and 2) does not yet have
         * an extent (ie table) initialized in the store manager.
         * <p>
         * After all of the table objects, including any other tables they might
         * reference, have been added, each table is initialized and validated
         * in the database.
         * </p>
         * <p>
         * If any error occurs along the way, any table(s) that were created are
         * dropped and the state of the RDBMSManager is rolled back to the point
         * at which this method was called.
         * </p>
         * @param classNames The class(es) whose tables are to be added.
         * @param clr the ClassLoaderResolver
         */
        private void addClassTablesAndValidate(String[] classNames, ClassLoaderResolver clr)
        {
            Map schemaDataByClassPrev = new HashMap(storeDataByClass);
            MultiMap schemaDataByClassAppIDPrev = new MultiMap(storeDataByAppIdClass);
            boolean completed = false;

            List tablesCreated = null;
            List tableConstraintsCreated = null;
            List autoCreateErrors = null;
            List viewsCreated = null;
            
            try
            {
                // Add SchemaData entries and Table's for the requested classes
                addClassTables(classNames, clr);

                List[] toValidate = initializeClassTables(classNames, clr);

                List[] result = performTablesValidation(toValidate[0], clr);
                tablesCreated = result[0];
                tableConstraintsCreated = result[1];
                autoCreateErrors = result[2];

                result = performViewsValidation(toValidate[1]);
                viewsCreated = result[0];
                autoCreateErrors.addAll(result[1]);

                verifyErrors(autoCreateErrors);

                completed = true;
            }
            catch (SQLException sqle)
            {
                String msg=LOCALISER_RDBMS.msg("RDBMS.Manager.AddClassesExceptionError", sqle);
                JPOXLogger.RDBMS_SCHEMA.error(msg);
                throw new JDODataStoreException(msg, sqle);
            }
            catch (Exception e)
            {
                if (JDOException.class.isAssignableFrom(e.getClass()))
                {
                    throw (JDOException) e;
                }
                else
                {
                    JPOXLogger.RDBMS_SCHEMA.error(LOCALISER_RDBMS.msg("RDBMS.Manager.AddClassesExceptionError", e));
                }
                throw new JDOFatalInternalException(e.toString(), e);
            }
            finally
            {
                // If something went wrong, roll things back to the way they
                // were before we started.
                // This may not restore the database 100% of the time
                // (if DDL statements are not transactional) but it will always
                // put the RDBMSManager's internal structures back the way they
                // were.
                if (!completed)
                {
                    rollbackSchemaCreation(schemaDataByClassPrev,schemaDataByClassAppIDPrev,viewsCreated,tableConstraintsCreated,tablesCreated);
                }
                schemaDataAdded.clear();
            }
        }

        /**
         * Called by Mapping objects in the midst of RDBMSManager.addClasses()
         * to request the creation of a join table to hold a containers' contents.
         * @param fmd The field metadata describing the Array field.
         * @param type The type of the join table
         */
        private DatastoreContainerObject addJoinTableForContainer(AbstractPropertyMetaData fmd, ClassLoaderResolver clr, int type)
        {
            DatastoreIdentifier tableName = getTableIdentifier(fmd, clr);
            DatastoreContainerObject join = null;
            if (type == JOIN_TABLE_SET)
            {
                join = new SetTable(tableName, fmd, RDBMSManager.this);
            }
            else if (type == JOIN_TABLE_LIST)
            {
                join = new ListTable(tableName, fmd, RDBMSManager.this);
            }
            else if (type == JOIN_TABLE_MAP)
            {
                join = new MapTable(tableName, fmd, RDBMSManager.this);
            }
            else if (type == JOIN_TABLE_ARRAY)
            {
                join = new ArrayTable(tableName, fmd, RDBMSManager.this);
            }

            RDBMSStoreData data;
            try
            {
                if (starter != null && starterInitialised && !starter.isOpen())
                {
                    starter.open();
                }

                data = new RDBMSStoreData(fmd, join);
                registerStoreData(data);
            }
            finally
            {
                if (starter != null && starterInitialised && starter.isOpen())
                {
                    starter.close();
                }
            }

            schemaDataAdded.add(data);
            return join;
        }
    }
}