/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.rdbms.columninfo;

import java.sql.ResultSet;
import java.sql.Types;

/**
 * Represents the metadata of a specific table column in DB2.
 * 
 * @version $Revision: 1.2 $
 */
public class DB2ColumnInfo extends ColumnInfo
{
    /**
     * Constructs a column information object from the current row of the given
     * result set.  The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getColumns().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from DatabaseMetaData.getColumns().
     */
    public DB2ColumnInfo(ResultSet rs)
    {
        super(rs);

        switch (dataType)
        {
            case Types.DATE:
            case Types.TIME:
            case Types.TIMESTAMP:
                /* Values > 0 inexplicably get returned here. */
                decimalDigits = 0;
                break;
            default:
                break;
        }
    }
}
