/**********************************************************************
Copyright (c) 2002 Mike Martin and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.rdbms.exceptions;

import javax.jdo.JDOFatalDataStoreException;

import org.jpox.store.rdbms.adapter.RDBMSAdapter;
import org.jpox.util.Localiser;

/**
 * A <tt>TooManyForeignKeysException</tt> is thrown when trying to add a foreign
 * key to a table and the table already has the maximum allowed number of
 * foreign keys.
 *
 * @version $Revision: 1.1 $ 
 */
public class TooManyForeignKeysException extends JDOFatalDataStoreException
{
    protected static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.rdbms.Localisation");

    /**
     * Constructs a too-many-foreign-keys exception.
     * @param dba the database adapter
     * @param table_name Name of the table with too many FKs
     */
    public TooManyForeignKeysException(RDBMSAdapter dba, String table_name)
    {
        super(LOCALISER.msg("RDBMS.Exception.TooManyForeignKeys","" + dba.getMaxForeignKeys(),table_name));
    }
}