/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.DB2TypeInfo;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Mapping of a Datalink RDBMS type (used by DB2).
 *
 * @version $Revision: 1.8 $
 **/
public class DatalinkRDBMSMapping extends CharRDBMSMapping
{
    /**
     * @param storeMgr Store Manager
     * @param mapping Java type mapping
     */
    protected DatalinkRDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }
    
    /**
     * Constructor.
     * @param mapping Java type mapping
     * @param storeMgr Store Manager
     * @param field Field to be mapped
     */
    public DatalinkRDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(mapping, storeMgr, field);
	}

    protected void initialize()
    {
        if (column != null)
        {
            if (mapping.getFieldMetaData().getValueForExtension("select-function") == null)
            {
                column.setWrapperFunction("DLURLCOMPLETEONLY(?)", Column.WRAPPER_FUNCTION_SELECT);
            }
        }
		initTypeInfo();
    }

    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(DB2TypeInfo.DATALINK);
    }    
    
    public String getInsertionInputParameter()
    {
        //instead of (?) we use (? || '') as workaround
        //could be replaced with something like ? CAST ( ? AS varchar(255) ) 
        return "DLVALUE(? || '')";
    }
    
    public boolean includeInFetchStatement()
    {
        return true;
    }

    public String getUpdateInputParameter()
    {
        //instead of (?) we use (? || '') as workaround
        //could be replaced with something like ? CAST ( ? AS varchar(255) ) 
        return "DLVALUE(? || '')";
    }	
}