/**********************************************************************
Copyright (c) 2006 Erik Bengtson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
2006 Andy Jefferson - commonised approach to BLOB writing as other Oracle mappings
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping.oracle;

import java.io.IOException;
import java.io.Serializable;
import java.sql.SQLException;

import javax.jdo.JDODataStoreException;

import org.jpox.StateManager;
import org.jpox.store.mapping.BitSetMapping;
import org.jpox.store.mapping.MappingCallbacks;
import org.jpox.store.rdbms.datatype.BlobImpl;
import org.jpox.util.TypeConversionHelper;

/**
 * Mapping for a BitSet type for Oracle.
 *
 * @version $Revision: 1.4 $
 **/
public class OracleBitSetMapping extends BitSetMapping implements MappingCallbacks
{
    /**
     * Method to be called after the insert of the owner class element.
     * @param sm StateManager of the owner
     **/
    public void postInsert(StateManager sm)
    {
        Object value = sm.provideField(fmd.getAbsoluteFieldNumber());
        if (value == null)
        {
            return;
        }

        // Generate the contents for the BLOB
        byte[] bytes = new byte[0];
        try
        {
            if (fmd.isSerialized())
            {
                // Serialised field so just perform basic Java serialisation for retrieval
                if (!(value instanceof Serializable))
                {
                    throw new JDODataStoreException(LOCALISER.msg("RDBMS.Mapping.ObjectNotSerializable", value.getClass().getName()));
                }
                BlobImpl b = new BlobImpl(value);
                bytes = b.getBytes(0, (int) b.length());
            }
            else if (value instanceof java.util.BitSet)
            {
                bytes = TypeConversionHelper.getByteArrayFromBooleanArray(TypeConversionHelper.getBooleanArrayFromBitSet((java.util.BitSet) value));
            }
            else
            {
                // Fall back to just perform Java serialisation for storage
                if (!(value instanceof Serializable))
                {
                    throw new JDODataStoreException(LOCALISER.msg("RDBMS.Mapping.ObjectNotSerializable", value.getClass().getName()));
                }
                BlobImpl b = new BlobImpl(value);
                bytes = b.getBytes(0, (int) b.length());
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMS.Mapping.UnableToSetParam", "Object", "" + value, fmd, e.getMessage()), e);
        }
        catch (IOException e1)
        {
            // Do nothing
        }

        // Update the BLOB
        OracleBlobRDBMSMapping.updateBlobColumn(sm, getDatastoreContainer(), getDataStoreMapping(0), bytes);
    }

    /**
     * Method to be called after any update of the owner class element.
     * @param sm StateManager of the owner
     */
    public void postUpdate(StateManager sm)
    {
        postInsert(sm);
    }

    public void deleteDependent(StateManager sm)
    {
    }

    public void postFetch(StateManager sm)
    {
    }

    public void preDelete(StateManager sm)
    {
    }
}