/**********************************************************************
Copyright (c) 2006 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 


Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping.oracle;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import javax.jdo.JDODataStoreException;
import javax.jdo.JDOObjectNotFoundException;
import javax.jdo.spi.PersistenceCapable;

import org.jpox.StateManager;
import org.jpox.state.StateManagerFactory;
import org.jpox.store.mapping.MappingCallbacks;
import org.jpox.store.mapping.SerialisedPCMapping;

/**
 * Mapping for a serialised PersistenceCapable object for Oracle.
 *
 * @version $Revision: 1.3 $
 */
public class OracleSerialisedPCMapping extends SerialisedPCMapping implements MappingCallbacks
{
    /**
     * @see org.jpox.store.mapping.MappingCallbacks#deleteDependent(org.jpox.StateManager)
     */
    public void deleteDependent(StateManager sm)
    {
    }

    /**
     * @see org.jpox.store.mapping.MappingCallbacks#postFetch(org.jpox.StateManager)
     */
    public void postFetch(StateManager sm)
    {
    }

    /**
     * Retrieve the empty BLOB created by the insert statement and write out the
     * current BLOB field value to the Oracle BLOB object
     * @param ownerSM the current StateManager
     * @throws JDOObjectNotFoundException
     * @throws JDODataStoreException
     */
    public void postInsert(StateManager ownerSM)
    {
        Object value = ownerSM.provideField(fmd.getAbsoluteFieldNumber());
        StateManager sm = null;
        if (value != null)
        {
            PersistenceCapable pc = (PersistenceCapable)value;
            sm = ownerSM.getPersistenceManager().findStateManager(pc);
            if (sm == null || pc.jdoGetPersistenceManager() == null)
            {
                // Assign a StateManager to the serialised object since none present
                sm = StateManagerFactory.newStateManager(pc, ownerSM.getPersistenceManager(), false);
                sm.addEmbeddedOwner(ownerSM, fmd.getAbsoluteFieldNumber());
            }
        }

        if (sm != null)
        {
            sm.setStoringPC();
        }

        // Generate the contents for the BLOB
        byte[] bytes = new byte[0];
        if (value != null)
        {
            try
            {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                ObjectOutputStream oos = new ObjectOutputStream(baos);
                oos.writeObject(value);
                bytes = baos.toByteArray();
            }
            catch (IOException e1)
            {
                // Do Nothing
            }
        }

        // Update the BLOB
        OracleBlobRDBMSMapping.updateBlobColumn(ownerSM, getDatastoreContainer(), getDataStoreMapping(0), bytes);

        if (sm != null)
        {
            sm.unsetStoringPC();
        }
    }

    /**
     * @see org.jpox.store.mapping.MappingCallbacks#postUpdate(org.jpox.StateManager)
     */
    public void postUpdate(StateManager sm)
    {
        postInsert(sm);
    }

    /**
     * @see org.jpox.store.mapping.MappingCallbacks#preDelete(org.jpox.StateManager)
     */
    public void preDelete(StateManager sm)
    {
    }
}