/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 
 

Contributors:
2003 Andy Jefferson - coding standards
2004 Andy Jefferson - change from Integer to Long
2004 Andy Jefferson - removed last-id option since was dangerous in concurrent apps
    ...
**********************************************************************/
package org.jpox.store.rdbms.poid;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

import org.jpox.store.poid.PoidBlock;
import org.jpox.store.poid.PoidException;
import org.jpox.store.rdbms.RDBMSManager;
import org.jpox.util.Localiser;

/**
 * This generator for Relational Databases uses the "select max(column) from
 * table" strategy. The block Poid size is limited to 1. MaxPoidGenerator works
 * with numbers, so clients using this Poid must cast the ID to Long *
 * <P>
 * <B>Required user properties</B>
 * </P>
 * <UL>
 * </UL>
 * <P>
 * <B>Optional user properties</B>
 * <UL>
 * </UL>
 * </P>
 *
 * @version $Revision: 1.8 $ 
 */
public class MaxPoidGenerator extends AbstractRDBMSPoidGenerator
{
    protected static final Localiser LOCALISER_RDBMS = Localiser.getInstance("org.jpox.store.rdbms.Localisation");

    /**
     * Constructor.
     * @param name Symbolic name for this generator
     * @param props Properties defining the behaviour of this generator
     */
    public MaxPoidGenerator(String name, Properties props)
    {
        super(name, props);
        allocationSize = 1;
    }

    /**
     * Method to reserve a block of identities.
     * Note : Only allocates a single id always.
     * @param size The block size
     * @return The reserved block
     */
    public PoidBlock reserveBlock(long size)
    {
        // search an Id in the database
        PreparedStatement ps = null;
        ResultSet rs = null;
        RDBMSManager rdbmsMgr = (RDBMSManager)storeMgr;
        try
        {
            String stmt = getStatement();
            ps = rdbmsMgr.getStatement(connection, stmt, false);

            rs = rdbmsMgr.executeStatementQuery(stmt, ps);
            if (!rs.next())
            {
                return new PoidBlock(new Object[] { new Long(1) });
            }

            return new PoidBlock(new Object[] { new Long(rs.getLong(1) + 1)});
        }
        catch (SQLException e)
        {
            //TODO adds a message correspondent to the exception. 
            //we need to work to create user friendly messages
            throw new PoidException(e.getMessage());
        }
        finally
        {
            try
            {
                if (rs != null)
                {
                    rs.close();
                }
                if (ps != null)
                {
                    ps.close();
                }
            }
            catch (SQLException e)
            {
                // no recoverable error
            }           
        }
    }

    /**
     * Return the SQL statement.
     * TODO Allow this to work in different catalog/schema
     * @return statement
     */
    private String getStatement()
    {
        StringBuffer stmt = new StringBuffer();
        stmt.append("SELECT max(");
        stmt.append(storeMgr.getIdentifierFactory().getIdentifierInAdapterCase((String)properties.get("column-name")));
        stmt.append(") FROM ");
        stmt.append(storeMgr.getIdentifierFactory().getIdentifierInAdapterCase((String)properties.get("table-name")));  
        return stmt.toString();
    }
}