/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
2002 Mike Martin (TJDO)
2003 Erik Bengtson - removed getObject
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/

package org.jpox.store.rdbms.query;


import java.sql.ResultSet;

import org.jpox.FetchPlanImpl;
import org.jpox.PersistenceManager;
import org.jpox.StateManager;
import org.jpox.store.FieldValues;
import org.jpox.store.StatementExpressionIndex;
import org.jpox.store.query.ResultObjectFactory;
import org.jpox.store.rdbms.fieldmanager.ResultSetGetter;

/**
 * Creates a Transient object instance from a ResultSet (JDBC).
 *
 * @version $Revision: 1.2.2.1 $
 **/
public final class TransientIDROF implements ResultObjectFactory
{
    private final Class candidateClass;
    protected final int[] fieldNumbers;
	protected final StatementExpressionIndex[] statementExpressionIndex;

	/**
     * Constructor 
     * @param candidateClass the candidate class
     * @param fieldNumbers the absolute field numbers with the retrieved fields
     * @param statementExpressionIndex the StatementExpressionIndex
	 */
    public TransientIDROF(Class candidateClass,
                          int[] fieldNumbers,
						  StatementExpressionIndex[] statementExpressionIndex)
    {
        this.candidateClass = candidateClass;
        this.fieldNumbers = fieldNumbers;
        this.statementExpressionIndex = statementExpressionIndex;
    }


	/* (non-Javadoc)
	 * @see org.jpox.store.Query.ResultObjectFactory#getObject(java.sql.ResultSet, java.lang.Class)
	 */
	public Object getObject(final PersistenceManager pm, final Object rs, Class pcClass)
    {
        return pm.getObjectById(pm.getStoreManager().newObjectID(pm,candidateClass.getName(),null),
            new FieldValues()
            {
                public void fetchFields(StateManager sm)
                {
                    sm.replaceFields(fieldNumbers, new ResultSetGetter(sm, (ResultSet)rs, statementExpressionIndex), false);
                }
                public void fetchNonLoadedFields(StateManager sm)
                {
                    sm.replaceNonLoadedFields(fieldNumbers, new ResultSetGetter(sm, (ResultSet)rs, statementExpressionIndex));
                }
                public FetchPlanImpl getFetchPlanForLoading()
                {
                    return null;
                }
            }
        );
	}
}