/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    Andy Jefferson - updated link to StoreManagerHelper
    Erik Bengtson - removed unused import
    ...
**********************************************************************/
package org.jpox.store.rdbms.typeinfo;

import java.sql.ResultSet;

import javax.jdo.JDOFatalDataStoreException;

/**
 * Represents the metadata of a specific exported key column.  This class is
 * basically a data structure that makes accessing the JDBC exported key
 * metadata easier.  Each of the items returned by
 * {@link java.sql.DatabaseMetaData#getExportedKeys(String,String,String)}
 * is represented by a public field in this class.
 *
 * Subclasses of ExportedKeyInfo can be created on a per-DBMS basis to supply
 * missing metadata or correct faulty metadata obtained from that DBMS's JDBC
 * driver(s).
 *
 * @version $Revision: 1.2 $ 
 */
public class ExportedKeyInfo extends ForeignKeyInfo
{

    /**
     * Constructs a exported key information object from the current row of the
     * given result set.
     * The {@link ResultSet} object passed must have been obtained from a call
     * to java.sql.DatabaseMetaData.getExportedKeys().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from
     *              java.sql.DatabaseMetaData.getImportedKeys() or
     *              java.sql.DatabaseMetaData.getExportedKeys().
     *
     * @exception JDOFatalDataStoreException
     *      if a column of foreign key information could not be retrieved from
     *      the result set.
     */
    public ExportedKeyInfo(ResultSet rs) throws JDOFatalDataStoreException
    {
    	super(rs);
    }
}