/**********************************************************************
Copyright (c) 2007 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms.typeinfo;

import java.sql.ResultSet;
import java.sql.Types;

import org.jpox.store.rdbms.columninfo.ColumnInfo;

/**
 * Represents the metadata of a H2 data type.
 *
 * @version $Revision: 1.2.2.2 $
 */
public class H2TypeInfo extends TypeInfo
{
    /**
     * Constructs a type information object from the current row of the given
     * result set. The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getTypeInfo().
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     * @param rs The result set returned from DatabaseMetaData.getTypeInfo().
     */
    public H2TypeInfo(ResultSet rs)
    {
        super(rs);
    }

    /**
     * Utility to check the compatibility of this type with the supplied Column
     * type.
     * @param colInfo The Column type
     * @return Whether they are compatible
     **/
    public boolean isCompatibleWith(ColumnInfo colInfo)
    {
        if (super.isCompatibleWith(colInfo))
        {
            return true;
        }

        if ((dataType == Types.CHAR && colInfo.getDataType() == Types.VARCHAR) ||
            (dataType == Types.VARCHAR && colInfo.getDataType() == Types.CHAR))
        {
            // H2 (at least til Feb 2007) returned VARCHAR after creating a col as CHAR so just call them the same
            return true;
        }

        return false;
    }
}