/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
2003 Erik Bengtson - type handling
    ...
**********************************************************************/
package org.jpox.store.rdbms.typeinfo;

import java.sql.ResultSet;
import java.sql.Types;

import org.jpox.store.rdbms.columninfo.ColumnInfo;

/**
 * Represents the metadata of a MSSQL data type.
 * 
 * @version $Revision: 1.3 $
 */
public class MSSQLTypeInfo extends TypeInfo
{
    /** sql type NVARCHAR **/
    public static final int NVARCHAR = -9;
    /** sql type NTEXT **/
    public static final int NTEXT = -10;
    /** sql type UNIQUEIDENTIFIER **/
    public static final int UNIQUEIDENTIFIER = -11;

    /**
     * Constructs a type information object from the current row of the given
     * result set.  The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getTypeInfo().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from DatabaseMetaData.getTypeInfo().
     */
    public MSSQLTypeInfo(ResultSet rs)
    {
        super(rs);
    }

    public boolean isCompatibleWith(ColumnInfo colInfo)
    {
        if (super.isCompatibleWith(colInfo))
            return true;

        switch (dataType)
        {
            case Types.VARCHAR:
                return colInfo.getDataType() == NVARCHAR;
            case Types.LONGVARCHAR:
                return colInfo.getDataType() == NTEXT;
            case Types.VARBINARY:
            case Types.LONGVARBINARY:
            case UNIQUEIDENTIFIER:
                return (colInfo.getDataType() == Types.VARBINARY) || (colInfo.getDataType() == Types.LONGVARBINARY) || (colInfo.getDataType() == UNIQUEIDENTIFIER);
            default:
                return false;
        }
    }
}