/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 

Contributors:
2003 Andy Jefferson - coding standards
2004 Andy Jefferson - fixed the use of LONGVARBINARY
    ...
**********************************************************************/
package org.jpox.store.rdbms.typeinfo;

import java.sql.ResultSet;
import java.sql.Types;

/**
 * Represents the metadata of a PostgreSQL data type.
 * 
 * @version $Revision: 1.3 $
 */
public class PostgreSQLTypeInfo extends TypeInfo
{
    /** The maximum precision we allow to be reported. */
    public static final int MAX_PRECISION = 65000;

    /**
     * Constructs a type information object from the current row of the given
     * result set.  The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getTypeInfo().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from DatabaseMetaData.getTypeInfo().
     */
    public PostgreSQLTypeInfo(ResultSet rs)
    {
        super(rs);

        /*
         * Not really sure what we should do about the precision field.  None of
         * the drivers that I've tried return a sensible value for it (the 7.2
         * driver returned -1 for all types, the 7.3 driver returned 9 (!) for
         * all types, both tried against a 7.2.1 server).  The best I can think
         * to do for now is make sure it has a decent value for those types
         * where I know JPOX cares, namely VARCHAR and NUMERIC.
         */

        if (typeName.equalsIgnoreCase("varchar") ||
            typeName.equalsIgnoreCase("char"))
        {
            /*
             * VARCHAR gets an arbitrary maximum precision assigned to it.
             * Requests fo anything larger than this will be converted to
             * LONGVARCHAR (i.e. TEXT) in StringMapping.
             */
            precision = MAX_PRECISION;
            
            // Question : What should the upper limit on CHAR be really ?
            // JDBC driver returns "9" !
        }
        else if (typeName.equalsIgnoreCase("numeric"))
        {
            precision = 64;    // pulled from thin air
        }
        else if (typeName.equalsIgnoreCase("text"))
        {
            // "text" is equated to Types.LONGVARCHAR
            dataType = Types.LONGVARCHAR;
        }
        else if (typeName.equalsIgnoreCase("bytea"))
        {
            // "bytea" is now equated to Types.LONGVARBINARY
            dataType = Types.LONGVARBINARY;
        }
        else if (typeName.equalsIgnoreCase("float8"))
        {
            allowsPrecisionSpec = false;
        }

        if (precision > MAX_PRECISION)
        {
            precision = MAX_PRECISION;
        }
    }
}